@@ -105,12 +105,12 @@ extern MODVAR ConfigItem_link		*conf_link;
 extern MODVAR ConfigItem_sni		*conf_sni;
 extern MODVAR ConfigItem_ban		*conf_ban;
 extern MODVAR ConfigItem_deny_channel  *conf_deny_channel;
-extern MODVAR ConfigItem_deny_link	*conf_deny_link;
 extern MODVAR ConfigItem_allow_channel *conf_allow_channel;
 extern MODVAR ConfigItem_deny_version	*conf_deny_version;
 extern MODVAR ConfigItem_alias		*conf_alias;
 extern MODVAR ConfigItem_help		*conf_help;
 extern MODVAR ConfigItem_offchans	*conf_offchans;
+extern MODVAR ConfigItem_proxy		*conf_proxy;
 extern void		completed_connection(int, int, void *);
 extern void clear_unknown();
 extern EVENT(e_unload_module_delayed);
@@ -149,7 +149,6 @@ extern ConfigItem_sni *find_sni(const char *name);
 extern ConfigItem_ulines	*find_uline(const char *host);
 extern ConfigItem_tld		*find_tld(Client *cptr);
 extern ConfigItem_link		*find_link(const char *servername);
-extern ConfigItem_deny_link *check_deny_link(ConfigItem_link *link, int auto_connect);
 extern ConfigItem_ban 		*find_ban(Client *, const char *host, short type);
 extern ConfigItem_ban 		*find_banEx(Client *,const char *host, short type, short type2);
 extern ConfigItem_vhost	*find_vhost(const char *name);
@@ -228,12 +227,15 @@ extern const char *extban_conv_param_nuh_or_extban(BanContext *b, Extban *extban
 extern const char *extban_conv_param_nuh(BanContext *b, Extban *extban);
 extern Ban *is_banned(Client *, Channel *, int, const char **, const char **);
 extern Ban *is_banned_with_nick(Client *, Channel *, int, const char *, const char **, const char **);
+extern int ban_exists(Ban *lst, const char *str);
+extern int ban_exists_ignore_time(Ban *lst, const char *str);
 
 extern Client *find_client(const char *, Client *);
 extern Client *find_name(const char *, Client *);
 extern Client *find_nickserv(const char *, Client *);
 extern Client *find_user(const char *, Client *);
 extern Client *find_server(const char *, Client *);
+extern Client *find_server_by_uid(const char *uid);
 extern Client *find_service(const char *, Client *);
 #define find_server_quick(x) find_server(x, NULL)
 extern char *find_or_add(char *);
@@ -257,7 +259,6 @@ extern MODVAR int readcalls, udpfd, resfd;
 extern Client *add_connection(ConfigItem_listen *, int);
 extern int check_server_init(Client *);
 extern void close_connection(Client *);
-extern void close_unbound_listeners();
 extern int get_sockerr(Client *);
 extern int inetport(ConfigItem_listen *, char *, int, int);
 extern void init_sys();
@@ -285,6 +286,7 @@ extern void sendto_channel(Channel *channel, Client *from, Client *skip,
 extern void sendto_local_common_channels(Client *user, Client *skip,
                                          long clicap, MessageTag *mtags,
                                          FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf,5,6)));
+extern void quit_sendto_local_common_channels(Client *user, MessageTag *mtags, const char *reason);
 extern void sendto_match_servs(Channel *, Client *, FORMAT_STRING(const char *), ...) __attribute__((format(printf,3,4)));
 extern void sendto_match_butone(Client *, Client *, const char *, int, MessageTag *,
     FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf,6,7)));
@@ -314,8 +316,33 @@ extern void sendnotice(Client *to, FORMAT_STRING(const char *pattern), ...) __at
  * @endcode
  * @ingroup SendFunctions
  */
-#define sendnumeric(to, numeric, ...) sendnumericfmt(to, numeric, STR_ ## numeric, ##__VA_ARGS__)
-extern void sendnumericfmt(Client *to, int numeric, FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf,3,4)));
+#define sendnumeric(to, numeric, ...) sendtaggednumericfmt(to, NULL, numeric, STR_ ## numeric, ##__VA_ARGS__)
+
+/** Send numeric message to a client - format to user specific needs.
+ * This will ignore the numeric definition of src/numeric.c and always send ":me.name numeric clientname "
+ * followed by the pattern and format string you choose.
+ * @param to		The recipient
+ * @param numeric	The numeric, one of RPL_* or ERR_*, see src/numeric.c
+ * @param pattern	The format string / pattern to use.
+ * @param ...		Format string parameters.
+ * @note Don't forget to add a colon if you need it (eg `:%%s`), this is a common mistake.
+ */
+#define sendnumericfmt(to, numeric, ...) sendtaggednumericfmt(to, NULL, numeric, __VA_ARGS__)
+
+/** Send numeric message to a client - format to user specific needs.
+ * This will ignore the numeric definition of src/numeric.c and always send ":me.name numeric clientname "
+ * followed by the pattern and format string you choose.
+ * @param to		The recipient
+ * @param mtags     NULL, or NULL-terminated array of message tags
+ * @param numeric	The numeric, one of RPL_* or ERR_*, see src/numeric.c
+ * @param pattern	The format string / pattern to use.
+ * @param ...		Format string parameters.
+ * @note Don't forget to add a colon if you need it (eg `:%%s`), this is a common mistake.
+ */
+#define sendtaggednumeric(to, mtags, numeric, ...) sendtaggednumericfmt(to, mtags, numeric, STR_ ## numeric, ##__VA_ARGS__)
+
+extern void sendtaggednumericfmt(Client *to, MessageTag *mtags, int numeric, FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf,4,5)));
+
 extern void sendtxtnumeric(Client *to, FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf,2,3)));
 /** Build numeric message so it is ready to be sent to a client - rarely used, normally you use sendnumeric() instead.
  * This function is normally only used in eg CAN_KICK and CAN_SET_TOPIC, where
@@ -405,12 +432,19 @@ extern void del_queries(const char *);
 #define CHAN_HASH_TABLE_SIZE 32768
 #define WHOWAS_HASH_TABLE_SIZE 32768
 #define THROTTLING_HASH_TABLE_SIZE 8192
+#define IPUSERS_HASH_TABLE_SIZE 8192
 extern uint64_t siphash(const char *in, const char *k);
 extern uint64_t siphash_raw(const char *in, size_t len, const char *k);
 extern uint64_t siphash_nocase(const char *in, const char *k);
 extern void siphash_generate_key(char *k);
 extern void init_hash(void);
-uint64_t hash_whowas_name(const char *name);
+extern void add_whowas_to_clist(WhoWas **, WhoWas *);
+extern void del_whowas_from_clist(WhoWas **, WhoWas *);
+extern void add_whowas_to_list(WhoWas **, WhoWas *);
+extern void del_whowas_from_list(WhoWas **, WhoWas *);
+extern uint64_t hash_whowas_name(const char *name);
+extern void create_whowas_entry(Client *client, WhoWas *e, WhoWasEvent event);
+extern void free_whowas_fields(WhoWas *e);
 extern int add_to_client_hash_table(const char *, Client *);
 extern int del_from_client_hash_table(const char *, Client *);
 extern int add_to_id_hash_table(const char *, Client *);
@@ -423,8 +457,12 @@ extern Client *hash_find_id(const char *, Client *);
 extern Client *hash_find_nickatserver(const char *, Client *);
 extern Channel *find_channel(const char *name);
 extern Client *hash_find_server(const char *, Client *);
-extern struct MODVAR ThrottlingBucket *ThrottlingHash[THROTTLING_HASH_TABLE_SIZE];
-
+extern IpUsersBucket *find_ipusers_bucket(Client *client);
+extern IpUsersBucket *add_ipusers_bucket(Client *client);
+extern void decrease_ipusers_bucket(Client *client);
+extern MODVAR struct ThrottlingBucket *ThrottlingHash[THROTTLING_HASH_TABLE_SIZE];
+extern MODVAR IpUsersBucket *IpUsersHash_ipv4[IPUSERS_HASH_TABLE_SIZE];
+extern MODVAR IpUsersBucket *IpUsersHash_ipv6[IPUSERS_HASH_TABLE_SIZE];
 
 
 /* Mode externs
@@ -494,11 +532,11 @@ extern MODVAR RealCommand *CommandHash[256];
 extern void init_CommandHash(void);
 
 /* CRULE */
-char *crule_parse(char *);
-int crule_test(char *);
-char *crule_errstring(int);
-int crule_eval(char *);
-void crule_free(char **);
+extern struct CRuleNode* crule_parse(const char*);
+extern void crule_free(struct CRuleNode**);
+extern int crule_eval(struct CRuleNode* rule);
+extern int crule_test(const char *rule);
+extern const char *crule_errstring(int errcode);
 
 /*
  * Close all local socket connections, invalidate client fd's
@@ -626,6 +664,8 @@ extern void *safe_alloc_sensitive(size_t size);
 /** Safely destroy a string in memory (but do not free!) */
 #define destroy_string(str) sodium_memzero(str, strlen(str))
 
+#define safe_json_decref(result)	do { json_decref(result); result = NULL; } while(0)
+
 /** @} */
 extern char *our_strdup(const char *str);
 extern char *our_strldup(const char *str, size_t max);
@@ -670,6 +710,7 @@ extern void IRCToRTF(unsigned char *buffer, unsigned char *string);
 extern void verify_opercount(Client *, const char *);
 extern int valid_host(const char *host, int strict);
 extern int valid_username(const char *username);
+extern int valid_vhost(const char *userhost);
 extern int count_oper_sessions(const char *);
 extern char *unreal_mktemp(const char *dir, const char *suffix);
 extern char *unreal_getpathname(const char *filepath, char *path);
@@ -748,7 +789,7 @@ extern MODVAR int (*can_join)(Client *client, Channel *channel, const char *key,
 extern MODVAR void (*do_mode)(Channel *channel, Client *client, MessageTag *mtags, int parc, const char *parv[], time_t sendts, int samode);
 extern MODVAR MultiLineMode *(*set_mode)(Channel *channel, Client *cptr, int parc, const char *parv[], u_int *pcount,
                             char pvar[MAXMODEPARAMS][MODEBUFLEN + 3]);
-extern MODVAR void (*set_channel_mode)(Channel *channel, const char *modes, const char *parameters);
+extern MODVAR void (*set_channel_mode)(Channel *channel, MessageTag *mtags, const char *modes, const char *parameters);
 extern MODVAR void (*set_channel_topic)(Client *client, Channel *channel, MessageTag *recv_mtags, const char *topic, const char *set_by, time_t set_at);
 extern MODVAR void (*cmd_umode)(Client *, MessageTag *, int, const char **);
 extern MODVAR int (*register_user)(Client *client);
@@ -820,6 +861,7 @@ extern MODVAR int (*do_remote_nick_name)(char *nick);
 extern MODVAR const char *(*charsys_get_current_languages)(void);
 extern MODVAR void (*broadcast_sinfo)(Client *acptr, Client *to, Client *except);
 extern MODVAR void (*connect_server)(ConfigItem_link *aconf, Client *by, struct hostent *hp);
+extern MODVAR int (*is_services_but_not_ulined)(Client *client);
 extern MODVAR void (*parse_message_tags)(Client *cptr, char **str, MessageTag **mtag_list);
 extern MODVAR const char *(*mtags_to_string)(MessageTag *m, Client *acptr);
 extern MODVAR int (*can_send_to_channel)(Client *cptr, Channel *channel, const char **msgtext, const char **errmsg, int notice);
@@ -863,6 +905,9 @@ extern MODVAR int (*websocket_handle_websocket)(Client *client, WebRequest *web,
 extern MODVAR int (*websocket_create_packet)(int opcode, char **buf, int *len);
 extern MODVAR int (*websocket_create_packet_ex)(int opcode, char **buf, int *len, char *sendbuf, size_t sendbufsize);
 extern MODVAR int (*websocket_create_packet_simple)(int opcode, const char **buf, int *len);
+extern MODVAR const char *(*check_deny_link)(ConfigItem_link *link, int auto_connect);
+extern MODVAR void (*mtag_add_issued_by)(MessageTag **mtags, Client *client, MessageTag *recv_mtags);
+extern MODVAR void (*cancel_ident_lookup)(Client *client);
 /* /Efuncs */
 
 /* TLS functions */
@@ -913,6 +958,8 @@ extern int websocket_handle_websocket_default_handler(Client *client, WebRequest
 extern int websocket_create_packet_default_handler(int opcode, char **buf, int *len);
 extern int websocket_create_packet_ex_default_handler(int opcode, char **buf, int *len, char *sendbuf, size_t sendbufsize);
 extern int websocket_create_packet_simple_default_handler(int opcode, const char **buf, int *len);
+extern void mtag_add_issued_by_default_handler(MessageTag **mtags, Client *client, MessageTag *recv_mtags);
+extern void cancel_ident_lookup_default_handler(Client *client);
 /* End of default handlers for efunctions */
 
 extern MODVAR MOTDFile opermotd, svsmotd, motd, botmotd, smotd, rules;
@@ -936,7 +983,6 @@ extern void unrealdns_delreq_bycptr(Client *cptr);
 extern void unrealdns_gethostbyname_link(const char *name, ConfigItem_link *conf, int ipv4_only);
 extern void unrealdns_delasyncconnects(void);
 extern EVENT(unrealdns_timeout);
-extern int is_autojoin_chan(const char *chname);
 extern void unreal_free_hostent(struct hostent *he);
 extern struct hostent *unreal_create_hostent(const char *name, const char *ip);
 extern const char *unreal_time_sync_error(void);
@@ -1026,6 +1072,7 @@ extern const char *pcre2_version(void);
 extern int get_terminal_width(void);
 extern int has_common_channels(Client *c1, Client *c2);
 extern int user_can_see_member(Client *user, Client *target, Channel *channel);
+extern int user_can_see_member_fast(Client *user, Client *target, Channel *channel, Member *target_member, const char *user_member_modes);
 extern int invisible_user_in_channel(Client *target, Channel *channel);
 extern MODVAR int tls_client_index;
 extern TLSOptions *FindTLSOptionsForUser(Client *acptr);
@@ -1037,7 +1084,7 @@ extern int verify_certificate(SSL *ssl, const char *hostname, char **errstr);
 extern const char *certificate_name(SSL *ssl);
 extern void start_of_normal_client_handshake(Client *acptr);
 extern void clicap_pre_rehash(void);
-extern void clicap_post_rehash(void);
+extern void clicap_check_for_changes(void);
 extern void unload_all_unused_mtag_handlers(void);
 extern void send_cap_notify(int add, const char *token);
 extern void sendbufto_one(Client *to, char *msg, unsigned int quick);
@@ -1045,6 +1092,7 @@ extern MODVAR int current_serial;
 extern const char *spki_fingerprint(Client *acptr);
 extern const char *spki_fingerprint_ex(X509 *x509_cert);
 extern int is_module_loaded(const char *name);
+extern int is_blacklisted_module(const char *name);
 extern void close_std_descriptors(void);
 extern void banned_client(Client *acptr, const char *bantype, const char *reason, int global, int noexit);
 extern char *mystpcpy(char *dst, const char *src);
@@ -1099,6 +1147,7 @@ extern int read_str(FILE *fd, char **x);
 extern void _free_entire_name_list(NameList *n);
 extern void _add_name_list(NameList **list, const char *name);
 extern void _del_name_list(NameList **list, const char *name);
+extern NameList *duplicate_name_list(NameList *e);
 extern NameList *find_name_list(NameList *list, const char *name);
 extern NameList *find_name_list_match(NameList *list, const char *name);
 extern int minimum_msec_since_last_run(struct timeval *tv_old, long minimum);
@@ -1157,6 +1206,7 @@ extern void add_nvplist_numeric_fmt(NameValuePrioList **lst, int priority, const
 extern NameValuePrioList *find_nvplist(NameValuePrioList *list, const char *name);
 extern const char *get_nvplist(NameValuePrioList *list, const char *name);
 extern void free_nvplist(NameValuePrioList *lst);
+extern NameValuePrioList *duplicate_nvplist(NameValuePrioList *e);
 extern void unreal_add_name_values(NameValuePrioList **n, const char *name, ConfigEntry *ce);
 extern const char *namevalue(NameValuePrioList *n);
 extern const char *namevalue_nospaces(NameValuePrioList *n);
@@ -1173,8 +1223,10 @@ extern void read_until(char **p, char *stopchars);
 extern int is_ip_valid(const char *ip);
 extern int is_file_readable(const char *file, const char *dir);
 /* json.c */
+extern int log_json_filter;
 extern json_t *json_string_unreal(const char *s);
 extern const char *json_object_get_string(json_t *j, const char *name);
+extern int json_object_get_integer(json_t *j, const char *name, int default_value);
 extern int json_object_get_boolean(json_t *j, const char *name, int default_value);
 extern json_t *json_timestamp(time_t v);
 extern const char *timestamp_iso8601_now(void);
@@ -1189,6 +1241,7 @@ extern void json_expand_tkl(json_t *j, const char *key, TKL *tkl, int detail);
 extern MODVAR SecurityGroup *securitygroups;
 extern void unreal_delete_masks(ConfigItem_mask *m);
 extern void unreal_add_masks(ConfigItem_mask **head, ConfigEntry *ce);
+extern ConfigItem_mask *unreal_duplicate_masks(ConfigItem_mask *existing);
 extern int unreal_mask_match(Client *acptr, ConfigItem_mask *m);
 extern int unreal_mask_match_string(const char *name, ConfigItem_mask *m);
 extern int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors);
@@ -1199,6 +1252,7 @@ extern int security_group_exists(const char *name);
 extern SecurityGroup *add_security_group(const char *name, int order);
 extern SecurityGroup *find_security_group(const char *name);
 extern void free_security_group(SecurityGroup *s);
+extern SecurityGroup *duplicate_security_group(SecurityGroup *s);
 extern void set_security_group_defaults(void);
 extern int user_allowed_by_security_group(Client *client, SecurityGroup *s);
 extern int user_allowed_by_security_group_name(Client *client, const char *secgroupname);
@@ -1208,6 +1262,8 @@ extern int conf_match_item(ConfigFile *conf, ConfigEntry *cep, SecurityGroup **b
 extern int test_match_block(ConfigFile *conf, ConfigEntry *ce, int *errors_out);
 extern int conf_match_block(ConfigFile *conf, ConfigEntry *ce, SecurityGroup **block);
 extern int test_extended_list(Extban *extban, ConfigEntry *cep, int *errors);
+extern int test_set_security_group(ConfigFile *conf, ConfigEntry *ce);
+extern int config_set_security_group(ConfigFile *conf, ConfigEntry *ce);
 /* securitygroup.c end */
 /* src/unrealdb.c start */
 extern UnrealDB *unrealdb_open(const char *filename, UnrealDBMode mode, char *secret_block);
@@ -1259,7 +1315,7 @@ extern const char *log_type_valtostring(LogType v);
 #endif
 extern void do_unreal_log(LogLevel loglevel, const char *subsystem, const char *event_id, Client *client, const char *msg, ...) __attribute__((format(printf,5,0)));
 extern void do_unreal_log_raw(LogLevel loglevel, const char *subsystem, const char *event_id, Client *client, const char *msg, ...);
-extern void do_unreal_log_internal_from_remote(LogLevel loglevel, const char *subsystem, const char *event_id, MultiLine *msg, const char *json_serialized, Client *from_server);
+extern void do_unreal_log_internal_from_remote(LogLevel loglevel, const char *subsystem, const char *event_id, MultiLine *msg, json_t *json, const char *json_serialized, Client *from_server);
 extern LogData *log_data_string(const char *key, const char *str);
 extern LogData *log_data_char(const char *key, const char c);
 extern LogData *log_data_integer(const char *key, int64_t integer);
@@ -1282,10 +1338,20 @@ extern const char *log_level_valtostring(LogLevel loglevel);
 extern LogLevel log_level_stringtoval(const char *str);
 extern int valid_event_id(const char *s);
 extern int valid_subsystem(const char *s);
+extern LogSource *add_log_source(const char *str);
+extern void free_log_sources(LogSource *l);
+extern int log_sources_match(LogSource *logsource, LogLevel loglevel, const char *subsystem, const char *event_id, int matched_already);
 extern const char *timestamp_iso8601_now(void);
 extern const char *timestamp_iso8601(time_t v);
 extern int is_valid_snomask(char c);
 extern int is_valid_snomask_string_testing(const char *str, char **wrong);
+extern MODVAR LogEntry *memory_log;
+extern MODVAR LogEntry *memory_log_tail;
+extern MODVAR int memory_log_entries;
+extern void free_memory_log_item(LogEntry *e);
+extern void memory_log_do_add_message(time_t t, LogLevel loglevel, const char *subsystem, const char *event_id, json_t *json);
+extern void memory_log_add_message(time_t t, LogLevel loglevel, const char *subsystem, const char *event_id, json_t *json);
+extern EVENT(memory_log_cleaner);
 /* end of logging */
 extern void add_fake_lag(Client *client, long msec);
 extern char *prefix_with_extban(const char *remainder, BanContext *b, Extban *extban, char *buf, size_t buflen);
@@ -1294,6 +1360,7 @@ extern GeoIPResult *geoip_lookup(const char *ip);
 extern void free_geoip_result(GeoIPResult *r);
 extern const char *get_operlogin(Client *client);
 extern const char *get_operclass(Client *client);
+extern struct sockaddr *raw_client_ip(Client *client);
 /* url stuff */
 extern const char *unreal_mkcache(const char *url);
 extern int has_cached_version(const char *url);
@@ -1302,6 +1369,7 @@ extern const char *displayurl(const char *url);
 extern char *url_getfilename(const char *url);
 extern void download_file_async(const char *url, time_t cachetime, vFP callback, void *callback_data, char *original_url, int maxredirects);
 extern void url_init(void);
+extern void url_cancel_handle_by_callback_data(void *ptr);
 extern EVENT(url_socket_timeout);
 /* end of url stuff */
 extern char *collapse(char *pattern);
@@ -1329,6 +1397,30 @@ extern void procio_post_rehash(int failure);
 /* end of proc i/o */
 extern int minimum_msec_since_last_run(struct timeval *tv_old, long minimum);
 extern long get_connected_time(Client *client);
+extern time_t get_creationtime(Client *client);
 extern const char *StripControlCodes(const char *text);
 extern const char *StripControlCodesEx(const char *text, char *output, size_t outputlen, int strip_flags);
 extern MODVAR Module *Modules;
+extern const char *command_issued_by_rpc(MessageTag *mtags);
+extern MODVAR int quick_close;
+extern MODVAR int connections_past_period;
+extern void deadsocket_exit(Client *client, int special);
+extern void close_listener(ConfigItem_listen *listener);
+extern int str_starts_with_case_sensitive(const char *haystack, const char *needle);
+extern int str_ends_with_case_sensitive(const char *haystack, const char *needle);
+extern int str_starts_with_case_insensitive(const char *haystack, const char *needle);
+extern int str_ends_with_case_insensitive(const char *haystack, const char *needle);
+extern void init_dynamic_set_block(DynamicSetBlock *s);
+extern void free_dynamic_set_block(DynamicSetBlock *s);
+extern int test_dynamic_set_block_item(ConfigFile *conf, const char *security_group, ConfigEntry *cep);
+extern int config_set_dynamic_set_block_item(ConfigFile *conf, DynamicSetBlock *s, ConfigEntry *cep);
+extern DynamicSetOption *get_setting_for_user(Client *client, SetOption opt);
+extern long long get_setting_for_user_number(Client *client, SetOption opt);
+extern const char *get_setting_for_user_string(Client *client, SetOption opt);
+extern void dynamic_set_string(DynamicSetBlock *s, int settingname, const char *value);
+extern void dynamic_set_number(DynamicSetBlock *s, int settingname, long long value);
+extern MODVAR DynamicSetBlock unknown_users_set;
+extern MODVAR DynamicSetBlock dynamic_set;
+extern void start_dns_and_ident_lookup(Client *client);
+extern void free_webserver(WebServer *webserver);
+#define safe_free_webserver(x)	do { if (x) { free_webserver(x); x = NULL; } } while(0)
