@@ -39,6 +39,9 @@
 #include <openssl/rand.h>
 #include <openssl/md5.h>
 #include <openssl/ripemd.h>
+#ifdef HAS_X509_check_host
+#include <openssl/x509v3.h>
+#endif
 #include <jansson.h>
 #include "common.h"
 #include "sys.h"
@@ -104,7 +107,6 @@ typedef struct ConfigItem_vhost ConfigItem_vhost;
 typedef struct ConfigItem_link	ConfigItem_link;
 typedef struct ConfigItem_ban ConfigItem_ban;
 typedef struct ConfigItem_deny_dcc ConfigItem_deny_dcc;
-typedef struct ConfigItem_deny_link ConfigItem_deny_link;
 typedef struct ConfigItem_deny_channel ConfigItem_deny_channel;
 typedef struct ConfigItem_deny_version ConfigItem_deny_version;
 typedef struct ConfigItem_alias ConfigItem_alias;
@@ -180,7 +182,10 @@ typedef OperPermission (*OperClassEntryEvalCallback)(OperClassACLEntryVar* varia
 #define	KEYLEN		23
 #define LINKLEN		32
 #define	BUFSIZE		512	/* WARNING: *DONT* CHANGE THIS!!!! */
-#define READBUFSIZE	8192	/* for the read buffer */
+#define MAXTAGSIZE	8192	/**< Maximum length of message tags (4K user + 4K server) */
+#define MAXLINELENGTH_USER	(MAXTAGSIZE+BUFSIZE)	/**< Maximum length of a line on IRC (for non-servers): 4k client tags + 4k server tags + 512 bytes (IRCv3) */
+#define MAXLINELENGTH	16384			/**< Maximum length of a line on IRC: from servers is 16k */
+#define READBUFSIZE	MAXLINELENGTH	/**< for the read buffer */
 #define	MAXRECIPIENTS 	20
 #define	MAXSILELENGTH	NICKLEN+USERLEN+HOSTLEN+10
 #define IDLEN		12
@@ -270,7 +275,9 @@ struct Log {
 	/* for destination::file */
 	char *file;
 	char *filefmt;
-	long maxsize;
+	long max_size;
+	int max_lines;
+	long long max_time;
 	int logfd;
 	/* for destination::channel */
 	int color;
@@ -279,8 +286,18 @@ struct Log {
 };
 
 /** This is used for deciding the <index> in logs[<index>] and temp_logs[<index>] */
-typedef enum LogDestination { LOG_DEST_SNOMASK=0, LOG_DEST_OPER=1, LOG_DEST_REMOTE=2, LOG_DEST_CHANNEL=3, LOG_DEST_DISK=4 } LogDestination;
-#define NUM_LOG_DESTINATIONS 5
+typedef enum LogDestination { LOG_DEST_SNOMASK=0, LOG_DEST_OPER=1, LOG_DEST_REMOTE=2, LOG_DEST_CHANNEL=3, LOG_DEST_DISK=4, LOG_DEST_MEMORY=5 } LogDestination;
+#define NUM_LOG_DESTINATIONS 6
+
+typedef struct LogEntry LogEntry;
+struct LogEntry {
+	LogEntry *prev, *next;
+	time_t t;
+	char loglevel;
+	char *subsystem;
+	char *event_id;
+	json_t *json;
+};
 
 /*
 ** 'offsetof' is defined in ANSI-C. The following definition
@@ -408,7 +425,9 @@ typedef enum ClientStatus {
 #define CLIENT_FLAG_NOHANDSHAKEDELAY	0x20000000	/**< No handshake delay */
 #define CLIENT_FLAG_SERVER_DISCONNECT_LOGGED	0x40000000	/**< Server disconnect message is (already) logged */
 #define CLIENT_FLAG_ASYNC_RPC			0x80000000	/**< Asynchronous remote RPC request - special case for rehash etc. */
-
+#define CLIENT_FLAG_IPUSERS_BUMPED	0x100000000	/**< The IpUsersBucket for this IP has been bumped (and needs to be decreased on disconnect) */
+#define CLIENT_FLAG_DEADSOCKET_IS_BANNED	0x200000000	/**< The deadsocket message should also send ERR_YOUREBANNEDCREEP and such */
+#define CLIENT_FLAG_CONNECT_FLOOD_CHECKED	0x400000000	/**< connect-flood has been checked (there are two hooks, so need this) */
 /** @} */
 
 #define OPER_SNOMASKS "+bBcdfkqsSoO"
@@ -431,6 +450,7 @@ typedef enum ClientStatus {
 #define PROTO_SJSBY	0x000020	/* SJOIN setby information (TS and nick) */
 #define PROTO_MTAGS	0x000040	/* Support message tags and big buffers */
 #define PROTO_NEXTBANS	0x000080	/* Server supports named extended bans */
+#define PROTO_BIGLINES	0x000100	/* BIGLINES support */
 
 /* For client capabilities: */
 #define CAP_INVERT	1L
@@ -567,7 +587,7 @@ typedef enum ClientStatus {
 #define ClearAsyncRPC(x)		do { (x)->flags &= ~CLIENT_FLAG_ASYNC_RPC; } while(0)
 /** @} */
 
-#define IsIPV6(x)			((x)->local->socket_type == SOCKET_TYPE_IPV6)
+#define IsIPV6(x)			((x)->local ? (((x)->local->socket_type == SOCKET_TYPE_IPV6) ? 1 : 0) : (strchr((x)->ip,':') ? 1 : 0))
 #define IsUnixSocket(x)			((x)->local->socket_type == SOCKET_TYPE_UNIX)
 #define SetIPV6(x)			do { (x)->local->socket_type = SOCKET_TYPE_IPV6; } while(0)
 #define SetUnixSocket(x)			do { (x)->local->socket_type = SOCKET_TYPE_UNIX; } while(0)
@@ -596,6 +616,7 @@ typedef enum ClientStatus {
 #define SupportCLK(x)		(CHECKSERVERPROTO(x, PROTO_CLK))
 #define SupportMTAGS(x)		(CHECKSERVERPROTO(x, PROTO_MTAGS))
 #define SupportNEXTBANS(x)	(CHECKSERVERPROTO(x, PROTO_NEXTBANS))
+#define SupportBIGLINES(x)	(CHECKSERVERPROTO(x, PROTO_BIGLINES))
 
 #define SetVL(x)		((x)->local->proto |= PROTO_VL)
 #define SetSJSBY(x)		((x)->local->proto |= PROTO_SJSBY)
@@ -603,6 +624,7 @@ typedef enum ClientStatus {
 #define SetCLK(x)		((x)->local->proto |= PROTO_CLK)
 #define SetMTAGS(x)		((x)->local->proto |= PROTO_MTAGS)
 #define SetNEXTBANS(x)		((x)->local->proto |= PROTO_NEXTBANS)
+#define SetBIGLINES(x)		((x)->local->proto |= PROTO_BIGLINES)
 
 /* Dcc deny types (see src/s_extra.c) */
 #define DCCDENY_HARD	0
@@ -850,10 +872,15 @@ typedef struct Whowas {
 	char *username;
 	char *hostname;
 	char *virthost;
+	char *ip;
 	char *servername;
 	char *realname;
+	char *account;
 	long umodes;
-	time_t   logoff;
+	time_t logon;
+	time_t logoff;
+	time_t connected_since;
+	WhoWasEvent event;
 	struct Client *online;	/* Pointer to new nickname for chasing or NULL */
 	struct Whowas *next;	/* for hash table... */
 	struct Whowas *prev;	/* for hash table... */
@@ -894,6 +921,8 @@ struct SWhois {
 #define CMD_OPER		0x0200
 /** Command is for control channel only (unrealircd.ctl socket) */
 #define CMD_CONTROL		0x0400
+/** Command is able to receive BIG lines */
+#define CMD_BIGLINES		0x0800
 
 /** Command function - used by all command handlers.
  * This is used in the code like <pre>CMD_FUNC(cmd_yourcmd)</pre> as a function definition.
@@ -1435,7 +1464,9 @@ typedef struct RPCClient RPCClient;
 /** RPC Client information */
 struct RPCClient {
 	char *rpc_user; /**< Name of the rpc-user block after authentication, NULL during pre-auth */
+	char *issuer; /**< Optional name of the issuer, set by rpc.set_issuer(), eg logged in user on admin panel, can be NULL */
 	json_t *rehash_request; /**< If a REHASH (request) is currently running, otherwise NULL */
+	LogSource *log_sources; /**< Subscribed to which log sources */
 };
 
 struct MessageTag {
@@ -1454,7 +1485,8 @@ typedef enum PreprocessorItem {
 
 typedef enum PreprocessorPhase {
 	PREPROCESSOR_PHASE_INITIAL	= 1,
-	PREPROCESSOR_PHASE_MODULE	= 2
+	PREPROCESSOR_PHASE_SECONDARY	= 2,
+	PREPROCESSOR_PHASE_MODULE	= 3
 } PreprocessorPhase;
 
 typedef enum AuthenticationType {
@@ -1482,6 +1514,30 @@ struct AuthConfig {
 #define crypt DES_crypt
 #endif
 
+/* CRULE stuff */
+
+#define CRULE_ALL		0
+#define CRULE_AUTO		1
+
+/* some constants and shared data types */
+#define CR_MAXARGLEN 80         /**< Maximum arg length (must be > HOSTLEN) */
+#define CR_MAXARGS 3            /**< Maximum number of args for a rule */
+
+/** Evaluation function for a connection rule. */
+typedef int (*crule_funcptr) (int, void **);
+
+/** CRULE - Node in a connection rule tree. */
+struct CRuleNode {
+  crule_funcptr funcptr; /**< Evaluation function for this node. */
+  int numargs;           /**< Number of arguments. */
+  void *arg[CR_MAXARGS]; /**< Array of arguments.  For operators, each arg
+                            is a tree element; for functions, each arg is
+                            a string. */
+};
+typedef struct CRuleNode CRuleNode;
+typedef struct CRuleNode* CRuleNodePtr;
+
+
 /*
  * conf2 stuff -stskeeps
 */
@@ -1559,9 +1615,6 @@ struct ConfigFlag_tld
 #define CONF_BAN_TYPE_AKILL	1
 #define CONF_BAN_TYPE_TEMPORARY 2
 
-#define CRULE_ALL		0
-#define CRULE_AUTO		1
-
 struct ConfigItem {
 	ConfigItem *prev, *next;
 	ConfigFlag flag;
@@ -1753,6 +1806,16 @@ typedef enum TransferEncoding {
 	TRANSFER_ENCODING_CHUNKED=1
 } TransferEncoding;
 
+/* Used to parse http Forwarded header (RFC 7239)... */
+#define IPLEN 48
+typedef struct HTTPForwardedHeader HTTPForwardedHeader;
+struct HTTPForwardedHeader
+{
+	int secure;
+	char hostname[HOSTLEN+1];
+	char ip[IPLEN+1];
+};
+
 typedef struct WebRequest WebRequest;
 struct WebRequest {
 	HttpMethod method; /**< GET/PUT/POST */
@@ -1770,6 +1833,7 @@ struct WebRequest {
 	long long chunk_remaining;
 	TransferEncoding transfer_encoding;
 	long long config_max_request_buffer_size; /**< CONFIG: Maximum request length allowed */
+	HTTPForwardedHeader *forwarded; /**< If using a proxy */
 };
 
 typedef struct WebServer WebServer;
@@ -1817,13 +1881,13 @@ struct ConfigItem_listen {
 	int options;			/**< e.g. LISTENER_BOUND if active */
 	int clients;			/**< Clients connected to this socket / listener */
 	int fd;				/**< File descriptor (if open), or -1 (if not open yet) */
+	char *spoof_ip;			/**< listen::spoof-ip (only for listen::file, if you want to override 127.0.0.1) */
 	SSL_CTX *ssl_ctx;		/**< SSL/TLS context */
 	TLSOptions *tls_options;	/**< SSL/TLS options */
 	WebServer *webserver;		/**< For the webserver module */
 	void (*start_handshake)(Client *client); /**< Function to call on accept() */
 	int websocket_options;		/**< Websocket options (for the websocket module) */
 	int rpc_options;		/**< For the RPC module */
-	char *websocket_forward;	/**< For websocket module too */
 };
 
 struct ConfigItem_sni {
@@ -1886,13 +1950,6 @@ struct ConfigItem_deny_dcc {
 	char			*filename, *reason;
 };
 
-struct ConfigItem_deny_link {
-	ConfigItem_deny_link *prev, *next;
-	ConfigFlag_except flag;
-	ConfigItem_mask  *mask;
-	char *rule, *prettyrule;
-};
-
 struct ConfigItem_deny_version {
 	ConfigItem_deny_version	*prev, *next;
 	ConfigFlag		flag;
@@ -1979,6 +2036,42 @@ struct ConfigItem_offchans {
 	char *topic;
 };
 
+typedef struct ConfigItem_proxy ConfigItem_proxy;
+
+typedef enum {
+	PROXY_WEBIRC_PASS=1,
+	PROXY_WEBIRC=2,
+	PROXY_WEB=3,
+} ProxyType;
+
+struct ConfigItem_proxy {
+	ConfigItem_proxy *prev, *next;
+	char *name;
+	SecurityGroup *mask;
+	ProxyType type;
+	AuthConfig *auth;
+};
+
+typedef union DynamicSetOption {
+	long long number;
+	char *string;
+} DynamicSetOption;
+
+typedef enum SetOption {
+	SET_AUTO_JOIN			= 0,	/**< set::auto-join */
+	SET_MODES_ON_CONNECT		= 1,	/**< set::modes-on-connect */
+	SET_RESTRICT_USERMODES		= 2,	/**< set::restrict-usermodes */
+	SET_MAX_CHANNELS_PER_USER	= 3,	/**< set::max-channels-per-user */
+	SET_STATIC_QUIT			= 4,	/**< set::static-quit */
+	SET_STATIC_PART			= 5	/**< set::static-part */
+} SetOption;
+#define MAXDYNAMICSETTINGS 16
+
+typedef struct DynamicSetBlock {
+	DynamicSetOption settings[MAXDYNAMICSETTINGS];
+	char isset[MAXDYNAMICSETTINGS];
+} DynamicSetBlock;
+
 #define SECURITYGROUPLEN 48
 struct SecurityGroup {
 	SecurityGroup *prev, *next;
@@ -1991,6 +2084,7 @@ struct SecurityGroup {
 	int reputation_score;
 	long connect_time;
 	int webirc;
+	int websocket;
 	int tls;
 	NameList *ip;
 	ConfigItem_mask *mask;
@@ -2001,11 +2095,14 @@ struct SecurityGroup {
 	int exclude_reputation_score;
 	long exclude_connect_time;
 	int exclude_webirc;
+	int exclude_websocket;
 	int exclude_tls;
 	NameList *exclude_ip;
 	ConfigItem_mask *exclude_mask;
 	NameList *exclude_security_group;
 	NameValuePrioList *exclude_extended;
+	/* Settings */
+	DynamicSetBlock settings;
 };
 
 #define HM_HOST 1
@@ -2243,6 +2340,15 @@ struct ThrottlingBucket
 	char count;
 };
 
+typedef struct IpUsersBucket IpUsersBucket;
+struct IpUsersBucket
+{
+	IpUsersBucket *prev, *next;
+	char rawip[16];
+	int local_clients;
+	int global_clients;
+};
+
 typedef struct CoreChannelModeTable CoreChannelModeTable;
 struct CoreChannelModeTable {
 	long mode;			/**< Mode value (which bit will be set) */
@@ -2370,6 +2476,7 @@ struct ConfigItem_badword {
 #define SEND_ALL	(SEND_LOCAL|SEND_REMOTE)
 #define SKIP_DEAF	0x4
 #define SKIP_CTCP	0x8
+#define CHECK_INVISIBLE	0x10
 
 typedef struct GeoIPResult GeoIPResult;
 struct GeoIPResult {
@@ -2420,6 +2527,16 @@ typedef enum JsonRpcError {
                                                          } \
                                                      } while(0)
 
+#define REQUIRE_PARAM_INTEGER(name, varname)         do { \
+                                                         json_t *t = json_object_get(params, name); \
+                                                         if (!t || !json_is_integer(t)) \
+                                                         { \
+                                                             rpc_error_fmt(client, request, JSON_RPC_ERROR_INVALID_PARAMS, "Missing parameter: '%s'", name); \
+                                                             return; \
+                                                         } \
+                                                         varname = json_integer_value(t); \
+                                                     } while(0)
+
 #define REQUIRE_PARAM_BOOLEAN(name, varname)         do { \
                                                          json_t *vvv = json_object_get(params, name); \
                                                          if (!v || !json_is_boolean(v)) \
@@ -2431,6 +2548,7 @@ typedef enum JsonRpcError {
                                                      } while(0)
 
 #define OPTIONAL_PARAM_STRING(name, varname)         varname = json_object_get_string(params, name)
+#define OPTIONAL_PARAM_INTEGER(name, varname, def)   varname = json_object_get_integer(params, name, def)
 #define OPTIONAL_PARAM_BOOLEAN(name, varname, def)   varname = json_object_get_boolean(params, name, def)
 
 #endif /* __struct_include__ */
