@@ -10,6 +10,9 @@
  * @brief JSON functions - used for logging and RPC.
  */
 
+/** Are we currently in the logging code? */
+int log_json_filter = 0;
+
 /** Calculate expansion of a JSON string thanks to double escaping.
  * orig => JSON => IRC
  *    " => \"   => \\"
@@ -47,8 +50,13 @@ json_t *json_string_unreal(const char *s)
 	if (s == NULL)
 		return json_null();
 
-	stripped = StripControlCodesEx(s, buf1, sizeof(buf1), UNRL_STRIP_LOW_ASCII|UNRL_STRIP_KEEP_LF);
-	verified_s = unrl_utf8_make_valid(buf1, buf2, sizeof(buf2), 0);
+	if (log_json_filter)
+	{
+		stripped = StripControlCodesEx(s, buf1, sizeof(buf1), UNRL_STRIP_LOW_ASCII|UNRL_STRIP_KEEP_LF);
+		verified_s = unrl_utf8_make_valid(buf1, buf2, sizeof(buf2), 0);
+	} else {
+		verified_s = unrl_utf8_make_valid(s, buf2, sizeof(buf2), 0);
+	}
 
 	return json_string(verified_s);
 }
@@ -59,6 +67,21 @@ const char *json_object_get_string(json_t *j, const char *name)
 	return v ? json_string_value(v) : NULL;
 }
 
+/** Get integer value of a JSON object.
+ * @param j		The JSON object that should contain a 'name' item
+ * @param name		The item to search for
+ * @param default_value	The value to return when the JSON object 'name' is not found
+ *			or not an integer.
+ * @returns The integer value, or default_value if the object does not exist or is not an integer.
+ */
+int json_object_get_integer(json_t *j, const char *name, int default_value)
+{
+	json_t *v = json_object_get(j, name);
+	if (!v || !json_is_integer(v))
+		return default_value;
+	return json_integer_value(v);
+}
+
 int json_object_get_boolean(json_t *j, const char *name, int default_value)
 {
 	json_t *v = json_object_get(j, name);
@@ -84,6 +107,16 @@ const char *json_get_value(json_t *t)
 		return buf;
 	}
 
+	if (json_is_boolean(t))
+	{
+		if (json_is_true(t))
+			return "true";
+		return "false";
+	}
+
+	if (json_is_array(t))
+		return "<array>";
+
 	return NULL;
 }
 
@@ -164,11 +197,18 @@ void json_expand_client_security_groups(json_t *parent, Client *client)
 			json_array_append_new(child, json_string_unreal(s->name));
 }
 
+/* detail=0:	only name, id
+ * detail=1:	only name, id, hostname, ip, details, geoip
+ * detail=2:	everything, except 'channels'
+ * detail=3:	everything, with 'channels' being a max 384 character string (meant for JSON logging only)
+ * detail=4:	everything, with 'channels' object (full).
+ */
 void json_expand_client(json_t *j, const char *key, Client *client, int detail)
 {
 	char buf[BUFSIZE+1];
 	json_t *child;
 	json_t *user = NULL;
+	time_t ts;
 
 	if (key)
 	{
@@ -179,10 +219,12 @@ void json_expand_client(json_t *j, const char *key, Client *client, int detail)
 	}
 
 	/* First the information that is available for ALL client types: */
-
 	json_object_set_new(child, "name", json_string_unreal(client->name));
 	json_object_set_new(child, "id", json_string_unreal(client->id));
 
+	if (detail == 0)
+		return;
+
 	/* hostname is available for all, it just depends a bit on whether it is DNS or IP */
 	if (client->user && *client->user->realhost)
 		json_object_set_new(child, "hostname", json_string_unreal(client->user->realhost));
@@ -193,11 +235,6 @@ void json_expand_client(json_t *j, const char *key, Client *client, int detail)
 
 	/* same for ip, is there for all (well, some services pseudo-users may not have one) */
 	json_object_set_new(child, "ip", json_string_unreal(client->ip));
-	if (client->local && client->local->listener)
-		json_object_set_new(child, "server_port", json_integer(client->local->listener->port));
-	if (client->local && client->local->port)
-		json_object_set_new(child, "client_port", json_integer(client->local->port));
-
 	/* client.details is always available: it is nick!user@host, nick@host, server@host
 	 * server@ip, or just server.
 	 */
@@ -215,9 +252,18 @@ void json_expand_client(json_t *j, const char *key, Client *client, int detail)
 		json_object_set_new(child, "details", json_string_unreal(client->name));
 	}
 
-	if (client->local && client->local->creationtime)
-		json_object_set_new(child, "connected_since", json_timestamp(client->local->creationtime));
+	if (detail < 2)
+	{
+		RunHook(HOOKTYPE_JSON_EXPAND_CLIENT, client, detail, child);
+		return;
+	}
 
+	if (client->local && client->local->listener)
+		json_object_set_new(child, "server_port", json_integer(client->local->listener->port));
+	if (client->local && client->local->port)
+		json_object_set_new(child, "client_port", json_integer(client->local->port));
+	if ((ts = get_creationtime(client)))
+		json_object_set_new(child, "connected_since", json_timestamp(ts));
 	if (client->local && client->local->idle_since)
 		json_object_set_new(child, "idle_since", json_timestamp(client->local->idle_since));
 
@@ -256,14 +302,20 @@ void json_expand_client(json_t *j, const char *key, Client *client, int detail)
 		str = get_operclass(client);
 		if (str)
 			json_object_set_new(user, "operclass", json_string_unreal(str));
-		if (client->user->channel)
+		/* For detail>2 we will include the channels.
+		 * Even if the user is on 0 channels we include "channels":[]
+		 * so it is clear that the user is on 0 channels and it is
+		 * not because of low detail level that channels are skipped.
+		 */
+		if (detail > 2)
 		{
 			Membership *m;
 			int cnt = 0;
 			int len = 0;
 			json_t *channels = json_array();
 			json_object_set_new(user, "channels", channels);
-			if (detail == 0)
+
+			if (detail == 3)
 			{
 				/* Short format, mainly for JSON logging */
 				for (m = client->user->channel; m; m = m->next)
@@ -396,6 +448,10 @@ void json_expand_channel_ban(json_t *child, const char *banlist_name, Ban *banli
 }
 
 
+/* detail=1 adds bans, ban_exemptions and invite_exceptions
+ * detail=2 adds members
+ * detail=3+ makes the members more detailed
+ */
 void json_expand_channel(json_t *j, const char *key, Channel *channel, int detail)
 {
 	char mode1[512], mode2[512], modes[512];
@@ -410,6 +466,9 @@ void json_expand_channel(json_t *j, const char *key, Channel *channel, int detai
 	}
 
 	json_object_set_new(child, "name", json_string_unreal(channel->name));
+	if (detail == 0)
+		return;
+
 	json_object_set_new(child, "creation_time", json_timestamp(channel->creationtime));
 	json_object_set_new(child, "num_users", json_integer(channel->users));
 	if (channel->topic)
@@ -436,7 +495,7 @@ void json_expand_channel(json_t *j, const char *key, Channel *channel, int detai
 		json_expand_channel_ban(child, "invite_exceptions", channel->invexlist);
 	}
 
-	if (detail > 2)
+	if (detail >= 3)
 	{
 		Member *u;
 		json_t *list = json_array();
@@ -445,10 +504,9 @@ void json_expand_channel(json_t *j, const char *key, Channel *channel, int detai
 		for (u = channel->members; u; u = u->next)
 		{
 			json_t *e = json_object();
-			json_object_set_new(e, "name", json_string_unreal(u->client->name));
-			json_object_set_new(e, "id", json_string_unreal(u->client->id));
 			if (*u->member_modes)
 				json_object_set_new(e, "level", json_string_unreal(u->member_modes));
+			json_expand_client(e, NULL, u->client, detail-3);
 			json_array_append_new(list, e);
 		}
 	}
