@@ -30,18 +30,26 @@
 // TODO: Make configurable at compile time (runtime won't do, as we haven't read the config file)
 #define show_event_console 0
 
+#define MAXLOGLENGTH 16384	/**< Maximum length of a log entry (which may be multiple lines) */
+
 /* Variables */
-Log *logs[NUM_LOG_DESTINATIONS] = { NULL, NULL, NULL, NULL, NULL };
-Log *temp_logs[NUM_LOG_DESTINATIONS] = { NULL, NULL, NULL, NULL, NULL };
+Log *logs[NUM_LOG_DESTINATIONS] = { NULL, NULL, NULL, NULL, NULL, NULL };
+Log *temp_logs[NUM_LOG_DESTINATIONS] = { NULL, NULL, NULL, NULL, NULL, NULL };
 static int snomask_num_destinations = 0;
 
 static char snomasks_in_use[257] = { '\0' };
 static char snomasks_in_use_testing[257] = { '\0' };
 
+LogEntry *memory_log = NULL; /**< Log entries in memory (OLDEST entry) */
+LogEntry *memory_log_tail = NULL; /**< Tail of log entries in memory (NEWEST entry) */
+int memory_log_entries = 0; /**< Number of memory_log entries */
+
 /* Forward declarations */
 int log_sources_match(LogSource *logsource, LogLevel loglevel, const char *subsystem, const char *event_id, int matched_already);
 void do_unreal_log_internal(LogLevel loglevel, const char *subsystem, const char *event_id, Client *client, int expand_msg, const char *msg, va_list vl);
 void log_blocks_switchover(void);
+void memory_log_add_message(time_t t, LogLevel loglevel, const char *subsystem, const char *event_id, json_t *json);
+EVENT(memory_log_cleaner);
 
 LogType log_type_stringtoval(const char *str)
 {
@@ -253,7 +261,7 @@ int config_test_log(ConfigFile *conf, ConfigEntry *block)
 								errors++;
 							}
 						} else
-						if (!strcmp(cepp->name, "maxsize"))
+						if (!strcmp(cepp->name, "maxsize") || !strcmp(cepp->name, "max-size"))
 						{
 							if (!cepp->value)
 							{
@@ -300,6 +308,35 @@ int config_test_log(ConfigFile *conf, ConfigEntry *block)
 						}
 					}
 				} else
+				if (!strcmp(cep->name, "memory"))
+				{
+					destinations++;
+					for (cepp = cep->items; cepp; cepp = cepp->next)
+					{
+						if (!strcmp(cepp->name, "max-lines"))
+						{
+							if (!cepp->value)
+							{
+								config_error_empty(cepp->file->filename,
+									cepp->line_number, "log", cepp->name);
+								errors++;
+							}
+						} else
+						if (!strcmp(cepp->name, "max-time"))
+						{
+							if (!cepp->value)
+							{
+								config_error_empty(cepp->file->filename,
+									cepp->line_number, "log", cepp->name);
+								errors++;
+							}
+						} else
+						{
+							config_error_unknown(cepp->file->filename, cepp->line_number, "log::destination::memory", cepp->name);
+							errors++;
+						}
+					}
+				} else
 				{
 					config_error_unknownopt(cep->file->filename, cep->line_number, "log::destination", cep->name);
 					errors++;
@@ -423,21 +460,25 @@ int config_run_log(ConfigFile *conf, ConfigEntry *block)
 				if (!strcmp(cep->name, "file") || !strcmp(cep->name, "syslog"))
 				{
 					Log *log;
-					/* First check if already exists... yeah this is a bit late
-					 * and ideally would have been done in config_test but...
-					 * that would have been lots of work for a (hopefully) rare case.
-					 */
-					for (log = temp_logs[LOG_DEST_DISK]; log; log = log->next)
+
+					if (!strcmp(cep->name, "file"))
 					{
-						if ((log->file && !strcmp(log->file, cep->value)) ||
-						    (log->filefmt && !strcmp(log->filefmt, cep->value)))
+						/* First check if already exists... yeah this is a bit late
+						 * and ideally would have been done in config_test but...
+						 * that would have been lots of work for a (hopefully) rare case.
+						 */
+						for (log = temp_logs[LOG_DEST_DISK]; log; log = log->next)
 						{
-							config_warn("%s:%d: Ignoring duplicate log block for file '%s'. "
-							            "You cannot have multiple log blocks logging to the same file.",
-							            cep->file->filename, cep->line_number,
-							            cep->value);
-							free_log_sources(sources);
-							return 0;
+							if ((log->file && !strcmp(log->file, cep->value)) ||
+							    (log->filefmt && !strcmp(log->filefmt, cep->value)))
+							{
+								config_warn("%s:%d: Ignoring duplicate log block for file '%s'. "
+									    "You cannot have multiple log blocks logging to the same file.",
+									    cep->file->filename, cep->line_number,
+									    cep->value);
+								free_log_sources(sources);
+								return 0;
+							}
 						}
 					}
 					log = safe_alloc(sizeof(Log));
@@ -452,9 +493,9 @@ int config_run_log(ConfigFile *conf, ConfigEntry *block)
 						safe_strdup(log->file, cep->value);
 					for (cepp = cep->items; cepp; cepp = cepp->next)
 					{
-						if (!strcmp(cepp->name, "maxsize"))
+						if (!strcmp(cepp->name, "maxsize") || !strcmp(cepp->name, "max-size"))
 						{
-							log->maxsize = config_checkval(cepp->value,CFG_SIZE);
+							log->max_size = config_checkval(cepp->value,CFG_SIZE);
 						}
 						else if (!strcmp(cepp->name, "type"))
 						{
@@ -462,6 +503,33 @@ int config_run_log(ConfigFile *conf, ConfigEntry *block)
 						}
 					}
 					AddListItem(log, temp_logs[LOG_DEST_DISK]);
+				} else
+				if (!strcmp(cep->name, "memory"))
+				{
+					if (temp_logs[LOG_DEST_MEMORY])
+					{
+						config_warn("%s:%d: Ignoring duplicate log block for memory. "
+							    "You cannot have multiple log blocks logging to memory.",
+							    cep->file->filename, cep->line_number);
+						free_log_sources(sources);
+						return 0;
+					}
+					Log *log = safe_alloc(sizeof(Log));
+					log->sources = sources;
+					log->logfd = -1;
+					log->type = LOG_TYPE_JSON;
+					for (cepp = cep->items; cepp; cepp = cepp->next)
+					{
+						if (!strcmp(cepp->name, "max-lines"))
+						{
+							log->max_lines = config_checkval(cepp->value,CFG_SIZE);
+						}
+						else if (!strcmp(cepp->name, "max-time"))
+						{
+							log->max_time = config_checkval(cepp->value,CFG_TIME);
+						}
+					}
+					AddListItem(log, temp_logs[LOG_DEST_MEMORY]);
 				}
 			}
 		}
@@ -878,9 +946,9 @@ literal:
 }
 
 /** Do the actual writing to log files */
-void do_unreal_log_disk(LogLevel loglevel, const char *subsystem, const char *event_id, MultiLine *msg, const char *json_serialized, Client *from_server)
+void do_unreal_log_disk(LogLevel loglevel, const char *subsystem, const char *event_id, MultiLine *msg, json_t *json, const char *json_serialized, Client *from_server)
 {
-	static int last_log_file_warning = 0;
+	static time_t last_log_file_warning = 0;
 	Log *l;
 	char timebuf[128];
 	struct stat fstats;
@@ -891,7 +959,8 @@ void do_unreal_log_disk(LogLevel loglevel, const char *subsystem, const char *ev
 
 	snprintf(timebuf, sizeof(timebuf), "[%s] ", myctime(TStime()));
 
-	RunHook(HOOKTYPE_LOG, loglevel, subsystem, event_id, msg, json_serialized, timebuf);
+	memory_log_add_message(TStime(), loglevel, subsystem, event_id, json);
+	RunHook(HOOKTYPE_LOG, loglevel, subsystem, event_id, msg, json, json_serialized, timebuf);
 
 	if (!loop.forked && (loglevel > ULOG_DEBUG))
 	{
@@ -965,7 +1034,7 @@ void do_unreal_log_disk(LogLevel loglevel, const char *subsystem, const char *ev
 		}
 
 		/* log::maxsize code */
-		if (l->maxsize && (stat(l->file, &fstats) != -1) && fstats.st_size >= l->maxsize)
+		if (l->max_size && (stat(l->file, &fstats) != -1) && fstats.st_size >= l->max_size)
 		{
 			char oldlog[512];
 			if (l->logfd == -1)
@@ -1039,7 +1108,7 @@ void do_unreal_log_disk(LogLevel loglevel, const char *subsystem, const char *ev
 		{
 			for (m = msg; m; m = m->next)
 			{
-				char text_buf[8192];
+				static char text_buf[MAXLOGLENGTH];
 				snprintf(text_buf, sizeof(text_buf), "%s%s %s.%s%s %s: %s\n",
 					timebuf, from_server->name,
 					subsystem, event_id, m->next?"+":"", log_level_valtostring(loglevel), m->line);
@@ -1426,11 +1495,14 @@ void do_unreal_log_internal(LogLevel loglevel, const char *subsystem, const char
 	json_t *j = NULL;
 	json_t *j_details = NULL;
 	json_t *t;
-	char msgbuf[8192];
+	char msgbuf[MAXLOGLENGTH];
 	const char *loglevel_string = log_level_valtostring(loglevel);
 	MultiLine *mmsg;
 	Client *from_server = NULL;
 
+	/* Set flag so json_string_unreal() uses more strict filter */
+	log_json_filter = 1;
+
 	if (loglevel_string == NULL)
 	{
 		do_unreal_log_norecursioncheck(ULOG_ERROR, "log", "BUG_LOG_LOGLEVEL", NULL,
@@ -1475,7 +1547,7 @@ void do_unreal_log_internal(LogLevel loglevel, const char *subsystem, const char
 	 * details later on.
 	 */
 	if (client)
-		json_expand_client(j_details, "client", client, 0);
+		json_expand_client(j_details, "client", client, 3);
 	/* Additional details (if any) */
 	while ((d = va_arg(vl, LogData *)))
 	{
@@ -1491,10 +1563,10 @@ void do_unreal_log_internal(LogLevel loglevel, const char *subsystem, const char
 					json_object_set_new(j_details, d->key, json_null());
 				break;
 			case LOG_FIELD_CLIENT:
-				json_expand_client(j_details, d->key, d->value.client, 0);
+				json_expand_client(j_details, d->key, d->value.client, 3);
 				break;
 			case LOG_FIELD_CHANNEL:
-				json_expand_channel(j_details, d->key, d->value.channel, 0);
+				json_expand_channel(j_details, d->key, d->value.channel, 1);
 				break;
 			case LOG_FIELD_OBJECT:
 				json_object_set_new(j_details, d->key, d->value.object);
@@ -1533,7 +1605,7 @@ void do_unreal_log_internal(LogLevel loglevel, const char *subsystem, const char
 
 	/* Now call all the loggers: */
 
-	do_unreal_log_disk(loglevel, subsystem, event_id, mmsg, json_serialized, from_server);
+	do_unreal_log_disk(loglevel, subsystem, event_id, mmsg, j, json_serialized, from_server);
 
 	if ((loop.rehashing == 2) || !strcmp(subsystem, "config"))
 		do_unreal_log_control(loglevel, subsystem, event_id, mmsg, j, json_serialized, from_server);
@@ -1546,27 +1618,52 @@ void do_unreal_log_internal(LogLevel loglevel, const char *subsystem, const char
 
 	// NOTE: code duplication further down!
 
+	/* This one should only be in do_unreal_log_internal()
+	 * and never in do_unreal_log_internal_from_remote()
+	 */
+	if (remote_rehash_client &&
+	    ((!strcmp(event_id, "CONFIG_ERROR_GENERIC") ||
+	      !strcmp(event_id, "CONFIG_WARNING_GENERIC") ||
+	      !strcmp(event_id, "CONFIG_INFO_GENERIC"))
+	     ||
+	     (loop.config_status >= CONFIG_STATUS_TEST)) &&
+	    strcmp(subsystem, "rawtraffic"))
+	{
+		sendto_log(remote_rehash_client, "NOTICE", remote_rehash_client->name,
+		           iConf.server_notice_colors, iConf.server_notice_show_event,
+		           loglevel, subsystem, event_id, mmsg, json_serialized, from_server);
+	}
+
 	/* Free everything */
 	safe_free(json_serialized);
 	safe_free_multiline(mmsg);
 	json_decref(j_details);
 	json_decref(j);
+
+	/* Turn off flag again */
+	log_json_filter = 0;
 }
 
 void do_unreal_log_internal_from_remote(LogLevel loglevel, const char *subsystem, const char *event_id,
-                                        MultiLine *msg, const char *json_serialized, Client *from_server)
+                                        MultiLine *msg, json_t *json, const char *json_serialized, Client *from_server)
 {
 	if (unreal_log_recursion_trap)
 		return;
 	unreal_log_recursion_trap = 1;
 
+	/* Set flag so json_string_unreal() uses more strict filter */
+	log_json_filter = 1;
+
 	/* Call the disk loggers */
-	do_unreal_log_disk(loglevel, subsystem, event_id, msg, json_serialized, from_server);
+	do_unreal_log_disk(loglevel, subsystem, event_id, msg, json, json_serialized, from_server);
 
 	/* And to IRC */
 	do_unreal_log_opers(loglevel, subsystem, event_id, msg, json_serialized, from_server);
 	do_unreal_log_channels(loglevel, subsystem, event_id, msg, json_serialized, from_server);
 
+	/* Turn off flag again */
+	log_json_filter = 0;
+
 	unreal_log_recursion_trap = 0;
 }
 
@@ -1592,7 +1689,7 @@ void free_log_block(Log *l)
 
 int log_tests(void)
 {
-	if (snomask_num_destinations <= 1)
+	if (snomask_num_destinations == 0)
 	{
 		unreal_log(ULOG_ERROR, "config", "LOG_SNOMASK_BLOCK_MISSING", NULL,
 		           "Missing snomask logging configuration:\n"
@@ -1600,7 +1697,6 @@ int log_tests(void)
 		           "include \"snomasks.default.conf\";");
 		return 0;
 	}
-	snomask_num_destinations = 0;
 	return 1;
 }
 
@@ -1621,7 +1717,7 @@ void postconf_defaults_log_block(void)
 	l = safe_alloc(sizeof(Log));
 	l->logfd = -1;
 	l->type = LOG_TYPE_TEXT; /* text */
-	l->maxsize = 100000000; /* maxsize 100M */
+	l->max_size = 100000000; /* maxsize 100M */
 	safe_strdup(l->file, "ircd.log");
 	convert_to_absolute_path(&l->file, LOGDIR);
 	AddListItem(l, logs[LOG_DEST_DISK]);
@@ -1649,6 +1745,7 @@ void postconf_defaults_log_block(void)
 void log_pre_rehash(void)
 {
 	*snomasks_in_use_testing = '\0';
+	snomask_num_destinations = 0;
 }
 
 /* Called after CONFIG_TEST right before CONFIG_RUN */
@@ -1708,3 +1805,102 @@ int is_valid_snomask_string_testing(const char *str, char **invalid_snomasks)
 	*invalid_snomasks = invalid_snomasks_buf;
 	return *invalid_snomasks_buf ? 0 : 1;
 }
+
+void free_memory_log_item(LogEntry *e)
+{
+	if (e == memory_log_tail)
+		memory_log_tail = memory_log_tail->prev;
+	DelListItem(e, memory_log);
+	safe_free(e->subsystem);
+	safe_free(e->event_id);
+	safe_json_decref(e->json);
+	safe_free(e);
+	memory_log_entries--;
+}
+
+/* IMPORTANT: this function only adds and never purges old entries */
+void memory_log_do_add_message(time_t t, LogLevel loglevel, const char *subsystem, const char *event_id, json_t *json)
+{
+	LogEntry *e;
+
+	e = safe_alloc(sizeof(LogEntry));
+	e->t = t;
+	e->loglevel = loglevel;
+	safe_strdup(e->subsystem, subsystem);
+	safe_strdup(e->event_id, event_id);
+	e->json = json_copy(json);
+
+	if (memory_log_tail)
+	{
+		/* We become the new tail */
+		memory_log_tail->next = e;
+		e->prev = memory_log_tail;
+		memory_log_tail = e;
+	} else
+	if (memory_log != memory_log_tail)
+	{
+		/* Impossible: memory_log NULL but memory_log_tail is not NULL, or vice versa */
+		abort();
+	} else
+	{
+		/* First memory log entry, easy */
+		memory_log = memory_log_tail = e;
+	}
+
+	memory_log_entries++;
+};
+
+void memory_log_add_message(time_t t, LogLevel loglevel, const char *subsystem, const char *event_id, json_t *json)
+{
+	Log *l;
+
+	for (l = logs[LOG_DEST_MEMORY]; l; l = l->next)
+	{
+		if (!strcmp(subsystem, "rawtraffic"))
+			continue;
+		if (!log_sources_match(l->sources, loglevel, subsystem, event_id, 0))
+			continue;
+		memory_log_do_add_message(t, loglevel, subsystem, event_id, json);
+		if (l->max_lines && (memory_log_entries > l->max_lines))
+			free_memory_log_item(memory_log);
+		break; /* we are done */
+	}
+}
+
+/** Purge old entries (by time and by line count) */
+EVENT(memory_log_cleaner)
+{
+	LogEntry *e, *e_next;
+	Log *l = logs[LOG_DEST_MEMORY];
+
+	if (!l)
+		return;
+
+	/* First, check if the hard limit has been reached */
+	if (l->max_lines)
+	{
+		int to_delete = memory_log_entries - l->max_lines;
+		if (to_delete > 0)
+		{
+			/* Delete the oldest ### entries */
+			for (e = memory_log; e; e = e_next)
+			{
+				e_next = e->next;
+				free_memory_log_item(e);
+			}
+		}
+	}
+
+	/* Now, erase by date/time */
+	if (l->max_time)
+	{
+		time_t deadline = TStime() - l->max_time;
+		/* Delete the entries before the deadline */
+		for (e = memory_log; e; e = e_next)
+		{
+			e_next = e->next;
+			if (e->t <= deadline)
+				free_memory_log_item(e);
+		}
+	}
+}
