@@ -471,7 +471,10 @@ static void exit_one_client(Client *client, MessageTag *mtags_i, const char *com
 			RunHook(HOOKTYPE_REMOTE_QUIT, client, mtags_i, comment);
 
 		new_message_special(client, mtags_i, &mtags_o, ":%s QUIT", client->name);
-		sendto_local_common_channels(client, NULL, 0, mtags_o, ":%s QUIT :%s", client->name, comment);
+		if (find_mtag(mtags_o, "unrealircd.org/real-quit-reason"))
+			quit_sendto_local_common_channels(client, mtags_o, comment);
+		else
+			sendto_local_common_channels(client, NULL, 0, mtags_o, ":%s QUIT :%s", client->name, comment);
 		free_message_tags(mtags_o);
 
 		while ((mp = client->user->channel))
@@ -750,6 +753,32 @@ int valid_username(const char *username)
 	return 1;
 }
 
+/** Check validity of a vhost which can be both in 'host' or 'user@host' format.
+ * This will call valid_username() and valid_host(xxx, 0) accordingly.
+ * @param userhost the "host" or "user@host"
+ * @returns 1 if valid, 0 if not.
+ */
+int valid_vhost(const char *userhost)
+{
+	char uhost[512], *p;
+	const char *host = userhost;
+
+        strlcpy(uhost, userhost, sizeof(uhost));
+
+	if ((p = strchr(uhost, '@')))
+	{
+		*p++ = '\0';
+		if (!valid_username(uhost))
+			return 0;
+		host = p;
+	}
+
+	if (!valid_host(host, 0))
+		return 0;
+
+	return 1;
+}
+
 /*|| BAN ACTION ROUTINES FOLLOW ||*/
 
 /** Converts a banaction string (eg: "kill") to an integer value (eg: BAN_ACT_KILL) */
@@ -908,33 +937,6 @@ const char *cmdname_by_spamftarget(int target)
 	return "???";
 }
 
-/** Returns 1 if this is a channel from set::auto-join or set::oper-auto-join */
-int is_autojoin_chan(const char *chname)
-{
-	char buf[512];
-	char *p, *name;
-
-	if (OPER_AUTO_JOIN_CHANS)
-	{
-		strlcpy(buf, OPER_AUTO_JOIN_CHANS, sizeof(buf));
-
-		for (name = strtoken(&p, buf, ","); name; name = strtoken(&p, NULL, ","))
-			if (!strcasecmp(name, chname))
-				return 1;
-	}
-
-	if (AUTO_JOIN_CHANS)
-	{
-		strlcpy(buf, AUTO_JOIN_CHANS, sizeof(buf));
-
-		for (name = strtoken(&p, buf, ","); name; name = strtoken(&p, NULL, ","))
-			if (!strcasecmp(name, chname))
-				return 1;
-	}
-
-	return 0;
-}
-
 /** Add name entries from config */
 void unreal_add_names(NameList **n, ConfigEntry *ce)
 {
@@ -1132,9 +1134,10 @@ void banned_client(Client *client, const char *bantype, const char *reason, int
 	char buf[512];
 	char *fmt = global ? iConf.reject_message_gline : iConf.reject_message_kline;
 	const char *vars[6], *values[6];
+	MessageTag *mtags = NULL;
 
 	if (!MyConnect(client))
-		abort(); /* hmm... or be more flexible? */
+		abort();
 
 	/* This was: "You are not welcome on this %s. %s: %s. %s" but is now dynamic: */
 	vars[0] = "bantype";
@@ -1162,29 +1165,32 @@ void banned_client(Client *client, const char *bantype, const char *reason, int
 	{
 		sendnumeric(client, ERR_YOUREBANNEDCREEP, buf);
 		sendnotice(client, "%s", buf);
-	} else {
-		send_raw_direct(client, ":%s %d %s :%s",
-		         me.name, ERR_YOUREBANNEDCREEP,
-		         (*client->name ? client->name : "*"),
-		         buf);
-		send_raw_direct(client, ":%s NOTICE %s :%s",
-		         me.name, (*client->name ? client->name : "*"), buf);
 	}
 
 	/* The final message in the ERROR is shorter. */
 	if (HIDE_BAN_REASON && IsRegistered(client))
-		snprintf(buf, sizeof(buf), "Banned (%s)", bantype);
-	else
+	{
+		/* Hide the ban reason, but put the real reason in unrealircd.org/real-quit-reason */
+		MessageTag *m = safe_alloc(sizeof(MessageTag));
+		safe_strdup(m->name, "unrealircd.org/real-quit-reason");
 		snprintf(buf, sizeof(buf), "Banned (%s): %s", bantype, reason);
+		safe_strdup(m->value, buf);
+		AddListItem(m, mtags);
+		/* And the quit reason for anyone else, goes here.. */
+		snprintf(buf, sizeof(buf), "Banned (%s)", bantype);
+	} else {
+		snprintf(buf, sizeof(buf), "Banned (%s): %s", bantype, reason);
+	}
 
 	if (noexit != NO_EXIT_CLIENT)
 	{
-		exit_client(client, NULL, buf);
+		exit_client(client, mtags, buf);
 	} else {
 		/* Special handling for direct Z-line code */
-		send_raw_direct(client, "ERROR :Closing Link: [%s] (%s)",
-		           client->ip, buf);
+		client->flags |= CLIENT_FLAG_DEADSOCKET_IS_BANNED;
+		dead_socket(client, buf);
 	}
+	safe_free_message_tags(mtags);
 }
 
 /** Our stpcpy implementation - discouraged due to lack of bounds checking */
@@ -1481,6 +1487,14 @@ int websocket_create_packet_simple_default_handler(int opcode, const char **buf,
 	return -1;
 }
 
+void mtag_add_issued_by_default_handler(MessageTag **mtags, Client *client, MessageTag *recv_mtags)
+{
+}
+
+void cancel_ident_lookup_default_handler(Client *client)
+{
+}
+
 /** my_timegm: mktime()-like function which will use GMT/UTC.
  * Strangely enough there is no standard function for this.
  * On some *NIX OS's timegm() may be available, sometimes only
@@ -2154,7 +2168,12 @@ int terminal_supports_color(void)
 #ifndef _WIN32
 	char *s;
 
-	/* Yeah we check all of stdin, stdout, stderr, because
+	/* Support NO_COLOR as per https://no-color.org */
+	s = getenv("NO_COLOR");
+	if (s != NULL && s[0] != '\0')
+		return 0;
+
+	/* Yeah we check all of stdin, stdout, stderr, because one
 	 * or more may be redirected (bin/unrealircd >log 2>&1),
 	 * and then we want to say no to color support.
 	 */
@@ -2510,6 +2529,7 @@ void server_reboot(const char *mesg)
 		WinExec(cmdLine, SW_SHOWDEFAULT);
 	}
 #endif
+	loop.terminating = 1;
 	unload_all_modules();
 #ifdef _WIN32
 	if (IsService)
@@ -2590,11 +2610,15 @@ const char *StripControlCodesEx(const char *text, char *output, size_t outputlen
 
 	while (len > 0) 
 	{
-		if ( col && ((isdigit(*text) && nc < 2) || (*text == ',' && nc < 3)))
+		if ((col && isdigit(*text) && nc < 2) ||
+		    ((col == 1) && (*text == ',') && isdigit(text[1]) && (nc > 0) && (nc < 3)))
 		{
 			nc++;
 			if (*text == ',')
+			{
 				nc = 0;
+				col++;
+			}
 		}
 		/* Syntax for RGB is ^DHHHHHH where H is a hex digit.
 		 * If < 6 hex digits are specified, the code is displayed
@@ -2697,3 +2721,11 @@ const char *StripControlCodes(const char *text)
 
 	return StripControlCodesEx(text, new_str, sizeof(new_str), 0);
 }
+
+const char *command_issued_by_rpc(MessageTag *mtags)
+{
+	MessageTag *m = find_mtag(mtags, "unrealircd.org/issued-by");
+	if (m && m->value && !strncmp(m->value, "RPC:", 4))
+		return m->value;
+	return NULL;
+}
