@@ -128,7 +128,7 @@ int _can_join(Client *client, Channel *channel, const char *key, char **errmsg)
 #ifndef NO_OPEROVERRIDE
 #ifdef OPEROVERRIDE_VERIFY
 	if (ValidatePermissionsForPath("channel:override:privsecret",client,NULL,channel,NULL) && (channel->mode.mode & MODE_SECRET ||
-	    channel->mode.mode & MODE_PRIVATE) && !is_autojoin_chan(channel->name))
+	    channel->mode.mode & MODE_PRIVATE))
 	{
 		*errmsg = STR_ERR_OPERSPVERIFY;
 		return (ERR_OPERSPVERIFY);
@@ -170,40 +170,25 @@ CMD_FUNC(cmd_join)
 }
 
 /** Send JOIN message for 'client' to all users in 'channel'.
- * Taking into account that not everyone in channel should see the JOIN (mode +D)
- * and taking into account the different types of JOIN (due to CAP extended-join).
+ * Taking into account the different types of JOIN (due to CAP extended-join).
  */
 void _send_join_to_local_users(Client *client, Channel *channel, MessageTag *mtags)
 {
-	int chanops_only = invisible_user_in_channel(client, channel);
-	Member *lp;
-	Client *acptr;
-	char joinbuf[512];
-	char exjoinbuf[512];
+	sendto_channel(channel, client, NULL, NULL,
+	               CAP_EXTENDED_JOIN|CAP_INVERT,
+	               CHECK_INVISIBLE|SEND_LOCAL,
+	               mtags,
+		       ":%s JOIN :%s",
+		       client->name, channel->name);
 
-	ircsnprintf(joinbuf, sizeof(joinbuf), ":%s!%s@%s JOIN :%s",
-		client->name, client->user->username, GetHost(client), channel->name);
-
-	ircsnprintf(exjoinbuf, sizeof(exjoinbuf), ":%s!%s@%s JOIN %s %s :%s",
-		client->name, client->user->username, GetHost(client), channel->name,
-		IsLoggedIn(client) ? client->user->account : "*",
-		client->info);
-
-	for (lp = channel->members; lp; lp = lp->next)
-	{
-		acptr = lp->client;
-
-		if (!MyConnect(acptr))
-			continue; /* only locally connected clients */
-
-		if (chanops_only && !check_channel_access_member(lp, "hoaq") && (client != acptr))
-			continue; /* skip non-ops if requested to (used for mode +D), but always send to 'client' */
-
-		if (HasCapabilityFast(acptr, CAP_EXTENDED_JOIN))
-			sendto_one(acptr, mtags, "%s", exjoinbuf);
-		else
-			sendto_one(acptr, mtags, "%s", joinbuf);
-	}
+	sendto_channel(channel, client, NULL, NULL,
+	               CAP_EXTENDED_JOIN,
+	               CHECK_INVISIBLE|SEND_LOCAL,
+	               mtags,
+		       ":%s JOIN %s %s :%s",
+		       client->name, channel->name,
+		       IsLoggedIn(client) ? client->user->account : "*",
+		       client->info);
 }
 
 /* Routine that actually makes a user join the channel
@@ -294,10 +279,13 @@ void _join_channel(Channel *channel, Client *client, MessageTag *recv_mtags, con
 
 		RunHook(HOOKTYPE_LOCAL_JOIN, client, channel, mtags);
 	} else {
-		unreal_log(ULOG_INFO, "join", "REMOTE_CLIENT_JOIN", client,
-			   "User $client joined $channel",
-			   log_data_channel("channel", channel),
-			   log_data_string("modes", member_modes));
+		if (!(client->uplink && !IsSynched(client->uplink)))
+		{
+			unreal_log(ULOG_INFO, "join", "REMOTE_CLIENT_JOIN", client,
+				   "User $client joined $channel",
+				   log_data_channel("channel", channel),
+				   log_data_string("modes", member_modes));
+		}
 		RunHook(HOOKTYPE_REMOTE_JOIN, client, channel, mtags);
 	}
 
@@ -458,12 +446,15 @@ void _do_join(Client *client, int parc, const char *parv[])
 			member_modes = (ChannelExists(name)) ? "" : LEVEL_ON_JOIN;
 
 			if (!ValidatePermissionsForPath("immune:maxchannelsperuser",client,NULL,NULL,NULL))	/* opers can join unlimited chans */
-				if (client->user->joined >= MAXCHANNELSPERUSER)
+			{
+				if (client->user->joined >= get_setting_for_user_number(client, SET_MAX_CHANNELS_PER_USER))
 				{
 					sendnumeric(client, ERR_TOOMANYCHANNELS, name);
 					RET();
 				}
-/* RESTRICTCHAN */
+			}
+
+			/* RESTRICTCHAN */
 			if (conf_deny_channel)
 			{
 				if (!ValidatePermissionsForPath("immune:server-ban:deny-channel",client,NULL,NULL,NULL))
@@ -492,7 +483,7 @@ void _do_join(Client *client, int parc, const char *parv[])
 					}
 				}
 			}
-			if (ValidatePermissionsForPath("immune:server-ban:deny-channel",client,NULL,NULL,NULL) && (tklban = find_qline(client, name, &ishold)))
+			if (!ValidatePermissionsForPath("immune:server-ban:deny-channel",client,NULL,NULL,NULL) && (tklban = find_qline(client, name, &ishold)))
 			{
 				sendnumeric(client, ERR_FORBIDDENCHANNEL, name, tklban->ptr.nameban->reason);
 				continue;
