@@ -56,6 +56,7 @@ CMD_FUNC(cmd_uid);
 int _register_user(Client *client);
 void nick_collision(Client *cptr, const char *newnick, const char *newid, Client *new, Client *existing, int type);
 int AllowClient(Client *client);
+int exceeds_maxperip(Client *client, ConfigItem_allow *aconf);
 
 MOD_TEST()
 {
@@ -122,9 +123,9 @@ CMD_FUNC(cmd_nick_remote)
 	{
 		ircstats.is_kill++;
 		unreal_log(ULOG_ERROR, "nick", "BAD_NICK_REMOTE", client,
-		           "Server link $server tried to introduce bad nick '$nick' -- rejected.",
+		           "Server link $server tried to change '$client' to bad nick '$nick' -- rejected.",
 		           log_data_string("nick", parv[1]),
-		           log_data_client("server", client->direction));
+		           log_data_client("server", client->uplink));
 		mtags = NULL;
 		new_message(client, NULL, &mtags);
 		sendto_one(client, mtags, ":%s KILL %s :Illegal nick name", me.id, client->id);
@@ -205,7 +206,7 @@ CMD_FUNC(cmd_nick_remote)
 	new_message(client, recv_mtags, &mtags);
 	RunHook(HOOKTYPE_REMOTE_NICKCHANGE, client, mtags, nick);
 	client->lastnick = lastnick ? lastnick : TStime();
-	add_history(client, 1);
+	add_history(client, 1, WHOWAS_EVENT_NICK_CHANGE);
 	sendto_server(client, 0, 0, mtags, ":%s NICK %s %lld",
 	    client->id, nick, (long long)client->lastnick);
 	sendto_local_common_channels(client, client, 0, mtags, ":%s NICK :%s", client->name, nick);
@@ -433,8 +434,8 @@ CMD_FUNC(cmd_nick_local)
 
 		new_message(client, recv_mtags, &mtags);
 		RunHook(HOOKTYPE_LOCAL_NICKCHANGE, client, mtags, nick);
-		client->lastnick = TStime();
-		add_history(client, 1);
+		add_history(client, 1, WHOWAS_EVENT_NICK_CHANGE);
+		client->lastnick = TStime(); /* needs to be done AFTER add_history() */
 		sendto_server(client, 0, 0, mtags, ":%s NICK %s %lld",
 		    client->id, nick, (long long)client->lastnick);
 		sendto_local_common_channels(client, client, 0, mtags, ":%s NICK :%s", client->name, nick);
@@ -698,6 +699,9 @@ nickkill2done:
 	if (*virthost != '*')
 		safe_strdup(client->user->virthost, virthost);
 
+	/* Add to ipusers hash table (to track global maxperip) */
+	exceeds_maxperip(client, NULL);
+
 	build_umode_string(client, 0, SEND_UMODES|UMODE_SERVNOTICE, buf);
 
 	sendto_serv_butone_nickcmd(client->direction, recv_mtags, client, (*buf == '\0' ? "+" : buf));
@@ -765,6 +769,7 @@ void welcome_user(Client *client, TKL *viruschan_tkl)
 {
 	int i;
 	ConfigItem_tld *tld;
+	char *chans;
 	char buf[BUFSIZE];
 
 	/* Make creation time the real 'online since' time, excluding registration time.
@@ -886,9 +891,9 @@ void welcome_user(Client *client, TKL *viruschan_tkl)
 		if (IsDead(client))
 			return;
 	}
-	else if (!BadPtr(AUTO_JOIN_CHANS) && strcmp(AUTO_JOIN_CHANS, "0"))
+	else if ((chans = (char *)get_setting_for_user_string(client, SET_AUTO_JOIN)) && strcmp(chans, "0"))
 	{
-		char *chans = strdup(AUTO_JOIN_CHANS);
+		chans = strdup(chans); // work on a copy (wait, is this still needed? we have a const guarantee now right?)
 		const char *args[3] = {
 			NULL,
 			chans,
@@ -1082,7 +1087,10 @@ int _register_user(Client *client)
 	SetUser(client);
 
 	make_cloakedhost(client, client->user->realhost, client->user->cloakedhost, sizeof(client->user->cloakedhost));
-	safe_strdup(client->user->virthost, client->user->cloakedhost);
+
+	/* client->user->virthost should never be empty */
+	if (!IsSetHost(client) || !client->user->virthost)
+		safe_strdup(client->user->virthost, client->user->cloakedhost);
 
 	snprintf(descbuf, sizeof descbuf, "Client: %s", client->name);
 	fd_desc(client->local->fd, descbuf);
@@ -1213,27 +1221,46 @@ void nick_collision(Client *cptr, const char *newnick, const char *newid, Client
 int exceeds_maxperip(Client *client, ConfigItem_allow *aconf)
 {
 	Client *acptr;
-	int local_cnt = 1;
-	int global_cnt = 1;
+	IpUsersBucket *bucket;
+
+	if (!client->ip)
+		return 0; /* eg. services */
+
+	bucket = find_ipusers_bucket(client);
+	if (!bucket)
+	{
+		client->flags |= CLIENT_FLAG_IPUSERS_BUMPED;
+		bucket = add_ipusers_bucket(client);
+		bucket->global_clients = 1;
+		if (MyConnect(client))
+			bucket->local_clients = 1;
+		return 0;
+	}
+
+	/* Bump if we haven't done so yet
+	 * (Actually not sure if this can ever be false, but...
+	 *  who knows with some 3rd party or some future change)
+	 */
+	if (!(client->flags & CLIENT_FLAG_IPUSERS_BUMPED))
+	{
+		bucket->global_clients++;
+		if (MyConnect(client))
+			bucket->local_clients++;
+		client->flags |= CLIENT_FLAG_IPUSERS_BUMPED;
+	}
 
 	if (find_tkl_exception(TKL_MAXPERIP, client))
 		return 0; /* exempt */
 
-	list_for_each_entry(acptr, &client_list, client_node)
+	if (aconf)
 	{
-		if (IsUser(acptr) && !strcmp(GetIP(acptr), GetIP(client)))
+		if ((bucket->local_clients > aconf->maxperip) ||
+		    (bucket->global_clients > aconf->global_maxperip))
 		{
-			if (MyUser(acptr))
-			{
-				local_cnt++;
-				if (local_cnt > aconf->maxperip)
-					return 1;
-			}
-			global_cnt++;
-			if (global_cnt > aconf->global_maxperip)
-				return 1;
+			return 1;
 		}
 	}
+
 	return 0;
 }
 
