@@ -10,12 +10,23 @@
 ModuleHeader MOD_HEADER
   = {
 	"rpc/rpc",
-	"1.0.2",
+	"1.0.4",
 	"RPC module for remote management",
 	"UnrealIRCd Team",
 	"unrealircd-6",
     };
 
+/** Maximum length of an rpc-user THIS { }.
+ * As we use the "RPC:" prefix it is nicklen minus that.
+ */
+#define RPCUSERLEN (NICKLEN-4)
+
+/** Timers can be minimum every <this> msec */
+#define RPC_MINIMUM_TIMER_MSEC 250
+
+#define RRPC_PACKET_SMALL	450
+#define RRPC_PACKET_BIGLINES	16000
+
 /* Structs */
 typedef struct RPCUser RPCUser;
 struct RPCUser {
@@ -44,6 +55,16 @@ struct OutstandingRRPC {
 	char *requestid;
 };
 
+typedef struct RPCTimer RPCTimer;
+struct RPCTimer {
+	RPCTimer *prev, *next;
+	long every_msec;
+	Client *client;
+	char *timer_id;
+	json_t *request;
+	struct timeval last_run;
+};
+
 /* Forward declarations */
 int rpc_config_test_listen(ConfigFile *cf, ConfigEntry *ce, int type, int *errs);
 int rpc_config_run_ex_listen(ConfigFile *cf, ConfigEntry *ce, int type, void *ptr);
@@ -61,7 +82,7 @@ int rpc_handle_body_websocket(Client *client, WebRequest *web, const char *readb
 int rpc_packet_in_websocket(Client *client, char *readbuf, int length);
 int rpc_packet_in_unix_socket(Client *client, const char *readbuf, int *length);
 void rpc_call_text(Client *client, const char *buf, int len);
-void rpc_call(Client *client, json_t *request);
+void rpc_call_json(Client *client, json_t *request);
 void _rpc_response(Client *client, json_t *request, json_t *result);
 void _rpc_error(Client *client, json_t *request, JsonRpcError error_code, const char *error_message);
 void _rpc_error_fmt(Client *client, json_t *request, JsonRpcError error_code, FORMAT_STRING(const char *fmt), ...) __attribute__((format(printf,4,5)));
@@ -73,17 +94,26 @@ int rpc_handle_auth(Client *client, WebRequest *web);
 int rpc_parse_auth_basic_auth(Client *client, WebRequest *web, char **username, char **password);
 int rpc_parse_auth_uri(Client *client, WebRequest *web, char **username, char **password);
 RPC_CALL_FUNC(rpc_rpc_info);
+RPC_CALL_FUNC(rpc_rpc_set_issuer);
+RPC_CALL_FUNC(rpc_rpc_add_timer);
+RPC_CALL_FUNC(rpc_rpc_del_timer);
 CMD_FUNC(cmd_rrpc);
 EVENT(rpc_remote_timeout);
+EVENT(rpc_do_timers);
 json_t *rrpc_data(RRPC *r);
 void free_rrpc_list(ModData *m);
 void free_outstanding_rrpc_list(ModData *m);
+void free_rpc_timer(RPCTimer *r);
+void free_rpc_timer_list(ModData *m);
 void rpc_call_remote(RRPC *r);
 void rpc_response_remote(RRPC *r);
+int rpc_handle_free_client(Client *client);
 int rpc_handle_server_quit(Client *client, MessageTag *mtags);
 int rpc_json_expand_client_server(Client *client, int detail, json_t *j, json_t *child);
 const char *rrpc_md_serialize(ModData *m);
 void rrpc_md_unserialize(const char *str, ModData *m);
+void rrpc_md_free(ModData *m);
+int rpc_reconfigure_web_listener(ConfigItem_listen *listener);
 
 /* Macros */
 #define RPC_PORT(client)  ((client->local && client->local->listener) ? client->local->listener->rpc_options : 0)
@@ -94,6 +124,7 @@ ModDataInfo *websocket_md = NULL; /* (imported) */
 RPCUser *rpcusers = NULL;
 RRPC *rrpc_list = NULL;
 OutstandingRRPC *outstanding_rrpc_list = NULL;
+RPCTimer *rpc_timer_list = NULL;
 ModDataInfo *rrpc_md;
 
 MOD_TEST()
@@ -130,7 +161,9 @@ MOD_INIT()
 	HookAdd(modinfo->handle, HOOKTYPE_PRE_LOCAL_HANDSHAKE_TIMEOUT, 0, rpc_pre_local_handshake_timeout);
 	HookAdd(modinfo->handle, HOOKTYPE_RAWPACKET_IN, INT_MIN, rpc_packet_in_unix_socket);
 	HookAdd(modinfo->handle, HOOKTYPE_SERVER_QUIT, 0, rpc_handle_server_quit);
+	HookAdd(modinfo->handle, HOOKTYPE_FREE_CLIENT, 0, rpc_handle_free_client);
 	HookAdd(modinfo->handle, HOOKTYPE_JSON_EXPAND_CLIENT_SERVER, 0, rpc_json_expand_client_server);
+	HookAdd(modinfo->handle, HOOKTYPE_RECONFIGURE_WEB_LISTENER, 0, rpc_reconfigure_web_listener);
 
 	memset(&r, 0, sizeof(r));
 	r.method = "rpc.info";
@@ -142,11 +175,42 @@ MOD_INIT()
 		return MOD_FAILED;
 	}
 
+	memset(&r, 0, sizeof(r));
+	r.method = "rpc.set_issuer";
+	r.loglevel = ULOG_DEBUG;
+	r.call = rpc_rpc_set_issuer;
+	if (!RPCHandlerAdd(modinfo->handle, &r))
+	{
+		config_error("[rpc.set_issuer] Could not register RPC handler");
+		return MOD_FAILED;
+	}
+
+	memset(&r, 0, sizeof(r));
+	r.method = "rpc.add_timer";
+	r.loglevel = ULOG_DEBUG;
+	r.call = rpc_rpc_add_timer;
+	if (!RPCHandlerAdd(modinfo->handle, &r))
+	{
+		config_error("[rpc.add_timer] Could not register RPC handler");
+		return MOD_FAILED;
+	}
+
+	memset(&r, 0, sizeof(r));
+	r.method = "rpc.del_timer";
+	r.loglevel = ULOG_DEBUG;
+	r.call = rpc_rpc_del_timer;
+	if (!RPCHandlerAdd(modinfo->handle, &r))
+	{
+		config_error("[rpc.del_timer] Could not register RPC handler");
+		return MOD_FAILED;
+	}
+
 	memset(&mreq, 0, sizeof(mreq));
 	mreq.name = "rrpc";
 	mreq.type = MODDATATYPE_CLIENT;
 	mreq.serialize = rrpc_md_serialize;
 	mreq.unserialize = rrpc_md_unserialize;
+	mreq.free = rrpc_md_free;
 	mreq.sync = 1;
 	mreq.self_write = 1;
 	rrpc_md = ModDataAdd(modinfo->handle, mreq);
@@ -158,10 +222,12 @@ MOD_INIT()
 
 	LoadPersistentPointer(modinfo, rrpc_list, free_rrpc_list);
 	LoadPersistentPointer(modinfo, outstanding_rrpc_list, free_outstanding_rrpc_list);
+	LoadPersistentPointer(modinfo, rpc_timer_list, free_rpc_timer_list);
 
-	CommandAdd(modinfo->handle, "RRPC", cmd_rrpc, MAXPARA, CMD_SERVER);
+	CommandAdd(modinfo->handle, "RRPC", cmd_rrpc, MAXPARA, CMD_SERVER|CMD_BIGLINES);
 
 	EventAdd(modinfo->handle, "rpc_remote_timeout", rpc_remote_timeout, NULL, 1000, 0);
+	EventAdd(modinfo->handle, "rpc_do_timers", rpc_do_timers, NULL, RPC_MINIMUM_TIMER_MSEC, 0);
 
 	/* Call MOD_LOAD very late, since we manage sockets, but depend on websocket_common */
 	ModuleSetOptions(modinfo->handle, MOD_OPT_PRIORITY, WEBSOCKET_MODULE_PRIORITY_UNLOAD-1);
@@ -207,6 +273,9 @@ void free_config(void)
 MOD_UNLOAD()
 {
 	free_config();
+	SavePersistentPointer(modinfo, rrpc_list);
+	SavePersistentPointer(modinfo, outstanding_rrpc_list);
+	SavePersistentPointer(modinfo, rpc_timer_list);
 	return MOD_SUCCESS;
 }
 
@@ -229,6 +298,19 @@ int rpc_config_test_listen(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 	return errors ? -1 : 1;
 }
 
+void rpc_listener_set_handler(ConfigItem_listen *l)
+{
+	if (l->socket_type == SOCKET_TYPE_UNIX)
+	{
+		l->start_handshake = rpc_client_handshake_unix_socket;
+	} else {
+		l->options |= LISTENER_TLS;
+		l->start_handshake = rpc_client_handshake_web;
+		l->webserver = safe_alloc(sizeof(WebServer));
+		l->webserver->handle_request = rpc_handle_webrequest;
+		l->webserver->handle_body = rpc_handle_webrequest_data;
+	}
+}
 int rpc_config_run_ex_listen(ConfigFile *cf, ConfigEntry *ce, int type, void *ptr)
 {
 	ConfigEntry *cep, *cepp;
@@ -243,21 +325,34 @@ int rpc_config_run_ex_listen(ConfigFile *cf, ConfigEntry *ce, int type, void *pt
 
 	l = (ConfigItem_listen *)ptr;
 	l->options |= LISTENER_NO_CHECK_CONNECT_FLOOD;
-	if (l->socket_type == SOCKET_TYPE_UNIX)
-	{
-		l->start_handshake = rpc_client_handshake_unix_socket;
-	} else {
-		l->options |= LISTENER_TLS;
-		l->start_handshake = rpc_client_handshake_web;
-		l->webserver = safe_alloc(sizeof(WebServer));
-		l->webserver->handle_request = rpc_handle_webrequest;
-		l->webserver->handle_body = rpc_handle_webrequest_data;
-	}
+	rpc_listener_set_handler(l);
 	l->rpc_options = 1;
 
 	return 1;
 }
 
+int rpc_reconfigure_web_listener(ConfigItem_listen *listener)
+{
+	if (listener->rpc_options)
+		rpc_listener_set_handler(listener);
+	return 0;
+}
+
+/** Valid name for rpc-user THISNAME { } ? */
+static int valid_rpc_user_name(const char *str)
+{
+	const char *p;
+
+	if (strlen(str) > RPCUSERLEN)
+		return 0;
+
+	for (p = str; *p; p++)
+		if (!isalnum(*p) && !strchr("_-", *p))
+			return 0;
+
+	return 1;
+}
+
 int rpc_config_test_rpc_user(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 {
 	int errors = 0;
@@ -272,9 +367,18 @@ int rpc_config_test_rpc_user(ConfigFile *cf, ConfigEntry *ce, int type, int *err
 	{
 		config_error("%s:%d: rpc-user block needs to have a name, eg: rpc-user apiuser { }",
 		             ce->file->filename, ce->line_number);
-		errors++;
+		*errs = 1;
+		return -1; /* quick return */
 	}
 
+	if (!valid_rpc_user_name(ce->value))
+	{
+		config_error("%s:%d: rpc-user block has invalid name '%s'. "
+		             "Can be max %d long and may only contain a-z, A-Z, 0-9, - and _.",
+		             ce->file->filename, ce->line_number,
+		             ce->value, RPCUSERLEN);
+		errors++;
+	}
 	for (cep = ce->items; cep; cep = cep->next)
 	{
 		if (!strcmp(cep->name, "match"))
@@ -458,13 +562,22 @@ int rpc_packet_in_websocket(Client *client, char *readbuf, int length)
 
 int rpc_packet_in_unix_socket(Client *client, const char *readbuf, int *length)
 {
+	char buf[READBUFSIZE];
+
 	if (!RPC_PORT(client) || !(client->local->listener->socket_type == SOCKET_TYPE_UNIX) || (*length <= 0))
 		return 1; /* Not for us */
 
-	// FIXME: this assumes a single request in 'readbuf' while in fact:
-	// - it could only contain partial JSON, eg no ending } yet
-	// - there could be multiple requests
-	rpc_call_text(client, readbuf, *length);
+	dbuf_put(&client->local->recvQ, readbuf, *length);
+
+	while (DBufLength(&client->local->recvQ))
+	{
+		int len = dbuf_getmsg(&client->local->recvQ, buf);
+		if (len <= 0)
+			break;
+		rpc_call_text(client, buf, len);
+		if (IsDead(client))
+			break;
+	}
 
 	return 0;
 }
@@ -478,18 +591,23 @@ void rpc_close(Client *client)
 }
 
 /** Handle the RPC request: input is a buffer with a certain length.
- * This calls rpc_call()
+ * This calls rpc_call_json()
  */
 void rpc_call_text(Client *client, const char *readbuf, int len)
 {
-	char buf[2048];
 	json_t *request = NULL;
 	json_error_t jerr;
+#if JANSSON_VERSION_HEX >= 0x020100
+	const char *buf = readbuf;
+	request = json_loadb(buf, len, JSON_REJECT_DUPLICATES, &jerr);
+#else
+	char buf[2048];
 
 	*buf = '\0';
 	strlncpy(buf, readbuf, sizeof(buf), len);
 
 	request = json_loads(buf, JSON_REJECT_DUPLICATES, &jerr);
+#endif
 	if (!request)
 	{
 		unreal_log(ULOG_INFO, "rpc", "RPC_INVALID_JSON", client,
@@ -500,12 +618,14 @@ void rpc_call_text(Client *client, const char *readbuf, int len)
 		rpc_close(client);
 		return;
 	}
-	rpc_call(client, request);
+	rpc_call_json(client, request);
 	json_decref(request);
 }
 
 void rpc_sendto(Client *client, const char *buf, int len)
 {
+	if (IsDead(client))
+		return;
 	if (MyConnect(client) && IsRPC(client) && WSU(client) && WSU(client)->handshake_completed)
 	{
 		/* Websocket */
@@ -689,63 +809,135 @@ int sanitize_params(Client *client, json_t *request, json_t *j)
 	return 1;
 }
 
-/** Handle the RPC request: request is in JSON */
-void rpc_call(Client *client, json_t *request)
+/** Log the RPC request */
+void rpc_call_log(Client *client, RPCHandler *handler, json_t *request, const char *method, json_t *params)
+{
+	const char *key;
+	json_t *value_object;
+	char params_string[512], tbuf[256];
+
+	*params_string = '\0';
+	json_object_foreach(params, key, value_object)
+	{
+		const char *value = json_get_value(value_object);
+		if (value)
+		{
+			snprintf(tbuf, sizeof(tbuf), "%s='%s', ", key, value);
+			strlcat(params_string, tbuf, sizeof(params_string));
+		}
+	}
+	if (*params_string)
+		params_string[strlen(params_string)-2] = '\0'; /* cut off last comma */
+
+	// TODO: pass log_data_json() or something, pass the entire 'request' ? For JSON logging
+
+	if (client->rpc && client->rpc->issuer)
+	{
+		if (*params_string)
+		{
+			unreal_log(handler->loglevel, "rpc", "RPC_CALL", client,
+				   "[rpc] RPC call $method by $client ($issuer): $params_string",
+				   log_data_string("issuer", client->rpc->issuer),
+				   log_data_string("method", method),
+				   log_data_string("params_string", params_string));
+		} else {
+			unreal_log(handler->loglevel, "rpc", "RPC_CALL", client,
+				   "[rpc] RPC call $method by $client ($issuer)",
+				   log_data_string("issuer", client->rpc->issuer),
+				   log_data_string("method", method));
+		}
+	} else {
+		if (*params_string)
+		{
+			unreal_log(handler->loglevel, "rpc", "RPC_CALL", client,
+				   "[rpc] RPC call $method by $client: $params_string",
+				   log_data_string("method", method),
+				   log_data_string("params_string", params_string));
+		} else {
+			unreal_log(handler->loglevel, "rpc", "RPC_CALL", client,
+				   "[rpc] RPC call $method by $client",
+				   log_data_string("method", method));
+		}
+	}
+}
+
+/** Parse an RPC request, except that it does not validate 'params'.
+ * @param client	The client issuing the request
+ * @param mainrequest	The underlying request that we should send errors to (usually same as 'request')
+ * @param request	The request that needs parsing
+ * @param method	This will be filled in if successfully parsed
+ * @param handler	This will be filled in if the handler is found
+ * @retval 0		An error occured while parsing or the method was not found.
+ * @retval 1		All good. You still need to validate 'params', though.
+ */
+int parse_rpc_call(Client *client, json_t *mainrequest, json_t *request, const char **method, RPCHandler **handler)
 {
-	json_t *t;
 	const char *jsonrpc;
-	const char *method;
-	const char *str;
 	json_t *id;
-	json_t *params;
-	RPCHandler *handler;
+	const char *str;
+
+	*method = NULL;
+	*handler = NULL;
 
 	jsonrpc = json_object_get_string(request, "jsonrpc");
 	if (!jsonrpc || strcasecmp(jsonrpc, "2.0"))
 	{
-		rpc_error(client, request, JSON_RPC_ERROR_INVALID_REQUEST, "Only JSON-RPC version 2.0 is supported");
-		return;
+		rpc_error(client, mainrequest, JSON_RPC_ERROR_INVALID_REQUEST, "Only JSON-RPC version 2.0 is supported");
+		return 0;
 	}
 
 	id = json_object_get(request, "id");
 	if (!id)
 	{
-		rpc_error(client, request, JSON_RPC_ERROR_INVALID_REQUEST, "Missing 'id'");
-		return;
+		rpc_error(client, mainrequest, JSON_RPC_ERROR_INVALID_REQUEST, "Missing 'id'");
+		return 0;
 	}
 
 	if ((str = json_string_value(id)))
 	{
 		if (strlen(str) > 32)
 		{
-			rpc_error(client, request, JSON_RPC_ERROR_INVALID_REQUEST, "The 'id' cannot be longer than 32 characters in UnrealIRCd JSON-RPC");
-			return;
+			rpc_error(client, mainrequest, JSON_RPC_ERROR_INVALID_REQUEST, "The 'id' cannot be longer than 32 characters in UnrealIRCd JSON-RPC");
+			return 0;
 		}
 		if (strchr(str, '\n') || strchr(str, '\r'))
 		{
-			rpc_error(client, request, JSON_RPC_ERROR_INVALID_REQUEST, "The 'id' may not contain \n or \r in UnrealIRCd JSON-RPC");
-			return;
+			rpc_error(client, mainrequest, JSON_RPC_ERROR_INVALID_REQUEST, "The 'id' may not contain \n or \r in UnrealIRCd JSON-RPC");
+			return 0;
 		}
 	} else if (!json_is_integer(id))
 	{
-		rpc_error(client, request, JSON_RPC_ERROR_INVALID_REQUEST, "The 'id' must be a string or an integer in UnrealIRCd JSON-RPC");
-		return;
+		rpc_error(client, mainrequest, JSON_RPC_ERROR_INVALID_REQUEST, "The 'id' must be a string or an integer in UnrealIRCd JSON-RPC");
+		return 0;
 	}
 
-	method = json_object_get_string(request, "method");
-	if (!method)
+	*method = json_object_get_string(request, "method");
+	if (!*method)
 	{
-		rpc_error(client, request, JSON_RPC_ERROR_INVALID_REQUEST, "Missing 'method' to call");
-		return;
+		rpc_error(client, mainrequest, JSON_RPC_ERROR_INVALID_REQUEST, "Missing 'method' to call");
+		return 0;
 	}
 
-	handler = RPCHandlerFind(method);
-	if (!handler)
+	*handler = RPCHandlerFind(*method);
+	if (!*handler)
 	{
-		rpc_error(client, request, JSON_RPC_ERROR_METHOD_NOT_FOUND, "Unsupported method");
-		return;
+		rpc_error(client, mainrequest, JSON_RPC_ERROR_METHOD_NOT_FOUND, "Unsupported method");
+		return 0;
 	}
 
+	return 1;
+}
+
+/** Handle the RPC request: request is in JSON */
+void rpc_call_json(Client *client, json_t *request)
+{
+	const char *method;
+	json_t *params;
+	RPCHandler *handler;
+
+	if (!parse_rpc_call(client, request, request, &method, &handler))
+		return; /* Error already returned to caller */
+
 	params = json_object_get(request, "params");
 	if (params)
 	{
@@ -763,9 +955,7 @@ void rpc_call(Client *client, json_t *request)
 		json_object_set_new(request, "params", params);
 	}
 
-	unreal_log(handler->loglevel, "rpc", "RPC_CALL", client,
-	           "[rpc] Client $client: RPC call $method",
-	           log_data_string("method", method));
+	rpc_call_log(client, handler, request, method, params);
 
 #ifdef DEBUGMODE
 	{
@@ -1011,6 +1201,31 @@ RPC_CALL_FUNC(rpc_rpc_info)
 	json_decref(result);
 }
 
+RPC_CALL_FUNC(rpc_rpc_set_issuer)
+{
+	json_t *result;
+	const char *name;
+	char buf[512];
+
+	REQUIRE_PARAM_STRING("name", name);
+
+	/* Do some validation on the name */
+	strlcpy(buf, name, sizeof(buf));
+	if (!do_remote_nick_name(buf) || strcmp(buf, name))
+	{
+		rpc_error(client, request, JSON_RPC_ERROR_INVALID_NAME,
+		          "The 'name' contains illegal characters or is too long. "
+		          "The same rules as for nick names apply.");
+		return;
+	}
+
+	safe_strdup(client->rpc->issuer, name);
+
+	result = json_boolean(1);
+	rpc_response(client, request, result);
+	json_decref(result);
+}
+
 void free_rrpc(RRPC *r)
 {
 	safe_free(r->requestid);
@@ -1050,6 +1265,52 @@ void free_outstanding_rrpc_list(ModData *m)
 	}
 }
 
+/** Remove timer from rpc_timer_list and free it */
+void free_rpc_timer(RPCTimer *r)
+{
+	safe_free(r->timer_id);
+	json_decref(r->request);
+	DelListItem(r, rpc_timer_list);
+	safe_free(r);
+}
+
+/* Admin unloading the RPC module for good (not called on rehash) */
+void free_rpc_timer_list(ModData *m)
+{
+	RPCTimer *r, *r_next;
+
+	for (r = rpc_timer_list; r; r = r_next)
+	{
+		r_next = r->next;
+		free_rpc_timer(r);
+	}
+}
+
+/* Admin unloading the RPC module for good (not called on rehash) */
+void free_rpc_timers_for_user(Client *client)
+{
+	RPCTimer *r, *r_next;
+
+	for (r = rpc_timer_list; r; r = r_next)
+	{
+		r_next = r->next;
+		if (r->client == client)
+			free_rpc_timer(r);
+	}
+}
+
+RPCTimer *find_rpc_timer(Client *client, const char *timer_id)
+{
+	RPCTimer *r;
+
+	for (r = rpc_timer_list; r; r = r->next)
+	{
+		if ((r->client == client) && !strcmp(timer_id, r->timer_id))
+			return r;
+	}
+	return NULL;
+}
+
 /** When a server quits, cancel all the RPC requests to/from those clients */
 int rpc_handle_server_quit(Client *client, MessageTag *mtags)
 {
@@ -1138,6 +1399,69 @@ OutstandingRRPC *find_outstandingrrpc(const char *source, const char *requestid)
 	return NULL;
 }
 
+void rrpc_pass_on_split(Client *client, Client *dest, MessageTag *recv_mtags, const char *parv[])
+{
+	char buf[MAXLINELENGTH];
+	char *data;
+	char saved;
+	char status[8];
+	char first = 0;
+	char continuation = 0;
+	char final = 0;
+	int remaining;
+
+	strlcpy(buf, parv[6], sizeof(buf));
+
+	if (strchr(parv[5], 'S'))
+		first = 1;
+	if (strchr(parv[5], 'C'))
+		continuation = 2;
+	if (strchr(parv[5], 'F'))
+		final = 1;
+
+	remaining = strlen(buf);
+	for (data = buf; remaining && *data; data += RRPC_PACKET_SMALL)
+	{
+		if (remaining > RRPC_PACKET_SMALL)
+		{
+			saved = data[RRPC_PACKET_SMALL];
+			data[RRPC_PACKET_SMALL] = '\0';
+			remaining -= RRPC_PACKET_SMALL;
+		} else {
+			saved = 0;
+			remaining = 0;
+			continuation = 0;
+		}
+
+		*status = '\0';
+		if (first)
+		{
+			strlcat_letter(status, 'S', sizeof(status));
+			first = 0;
+		}
+		if (!saved && final)
+		{
+			/* Send 'F' - but not just when its the last chunk,
+			 * only if the original also indicated it is the
+			 * last chunk (caveat :D).
+			 */
+			strlcat_letter(status, 'F', sizeof(status));
+		} else
+		if (!first)
+		{
+			strlcat_letter(status, 'C', sizeof(status));
+		}
+
+		sendto_one(dest, recv_mtags, ":%s RRPC %s %s %s %s %s :%s",
+			   client->id, parv[1], parv[2], parv[3], parv[4], status, data);
+
+		if (!saved)
+			break; /* done! */
+
+		data[RRPC_PACKET_SMALL] = saved;
+	}
+}
+
 /* Remote RPC call over the network (RRPC)
  * :<server> RRPC <REQ|RES> <source> <destination> <requestid> [S|C|F] :<request data>
  * S = Start
@@ -1156,7 +1480,7 @@ CMD_FUNC(cmd_rrpc)
 
 	if ((parc < 7) || BadPtr(parv[6]))
 	{
-		sendnumeric(client, ERR_NEEDMOREPARAMS, "KNOCK");
+		sendnumeric(client, ERR_NEEDMOREPARAMS, "RRPC");
 		return;
 	}
 
@@ -1178,7 +1502,7 @@ CMD_FUNC(cmd_rrpc)
 	data = parv[6];
 
 	/* Search by SID (first 3 characters of destination)
-	 * so we can always deliver, even forn unknown UID destinations
+	 * so we can always deliver, even for unknown UID destinations
 	 * in case this is a response.
 	 */
 	strlcpy(sid, destination, sizeof(sid));
@@ -1191,9 +1515,16 @@ CMD_FUNC(cmd_rrpc)
 
 	if (dest != &me)
 	{
-		/* Just pass it along... */
-		sendto_one(dest, recv_mtags, ":%s RRPC %s %s %s %s %s :%s",
-		           client->id, parv[1], parv[2], parv[3], parv[4], parv[5], parv[6]);
+		/* Not for us, pass it along... */
+		if ((strlen(parv[6]) > RRPC_PACKET_SMALL) && !SupportBIGLINES(dest->direction))
+		{
+			/* Hard case */
+			rrpc_pass_on_split(client, dest, recv_mtags, parv);
+		} else {
+			/* Simple case */
+			sendto_one(dest, recv_mtags, ":%s RRPC %s %s %s %s %s :%s",
+			           client->id, parv[1], parv[2], parv[3], parv[4], parv[5], parv[6]);
+		}
 		return;
 	}
 
@@ -1296,7 +1627,7 @@ void rpc_call_remote(RRPC *r)
 	safe_strdup(client->rpc->rpc_user, "<remote>");
 	// Note: NOT added to hash table or id table etc.
 	list_add(&client->client_node, &rpc_remote_list);
-	rpc_call(client, request);
+	rpc_call_json(client, request);
 	json_decref(request);
 
 	/* And free the temporary client, unless it is async... */
@@ -1376,7 +1707,8 @@ void rpc_send_generic_to_remote(Client *source, Client *target, const char *requ
 	int bytes; /* bytes in this frame */
 	int bytes_remaining; /* bytes remaining overall */
 	int start_frame = 1; /* set to 1 if this is the start frame */
-	char data[451];
+	int packet_split_size; /* chunk size of outgoing packets (depends on BIGLINES support) */
+	char data[RRPC_PACKET_BIGLINES+1];
 
 	requestid = rpc_id(json);
 	if (!requestid)
@@ -1392,10 +1724,15 @@ void rpc_send_generic_to_remote(Client *source, Client *target, const char *requ
 	 * F = Finish
 	 */
 
+	if (SupportBIGLINES(target->direction))
+		packet_split_size = RRPC_PACKET_BIGLINES;
+	else
+		packet_split_size = RRPC_PACKET_SMALL;
+
 	bytes_remaining = strlen(json_serialized);
-	for (str = json_serialized, bytes = MIN(bytes_remaining, 450);
+	for (str = json_serialized, bytes = MIN(bytes_remaining, packet_split_size);
 	     str && *str && bytes_remaining;
-	     str += bytes, bytes = MIN(bytes_remaining, 450))
+	     str += bytes, bytes = MIN(bytes_remaining, packet_split_size))
 	{
 		bytes_remaining -= bytes;
 		if (start_frame == 1)
@@ -1413,7 +1750,7 @@ void rpc_send_generic_to_remote(Client *source, Client *target, const char *requ
 			type = "F"; /* finish frame (the last frame) */
 		}
 
-		strlncpy(data, str, sizeof(data), bytes);
+		strlncpy(data, str, packet_split_size+1, bytes);
 
 		sendto_one(target, NULL, ":%s RRPC %s %s %s %s %s :%s",
 		           me.id,
@@ -1545,12 +1882,21 @@ void rrpc_md_unserialize(const char *str, ModData *m)
 	}
 }
 
+void rrpc_md_free(ModData *m)
+{
+	if (m->ptr)
+	{
+		free_nvplist(m->ptr);
+		m->ptr = NULL;
+	}
+}
+
 int rpc_json_expand_client_server(Client *client, int detail, json_t *j, json_t *child)
 {
 	NameValuePrioList *nv = RRPCMODULES(client);
 	json_t *rpc_modules;
 
-	if (!nv || (detail < 1))
+	if (!nv || (detail < 2))
 		return 0;
 
 	/* All this belongs under 'features' */
@@ -1569,3 +1915,100 @@ int rpc_json_expand_client_server(Client *client, int detail, json_t *j, json_t
 	}
 	return 0;
 }
+
+RPC_CALL_FUNC(rpc_rpc_add_timer)
+{
+	json_t *result;
+	json_t *subrequest;
+	long every_msec;
+	const char *timer_id;
+	const char *method;
+	RPCHandler *handler;
+	RPCTimer *timer;
+
+	REQUIRE_PARAM_INTEGER("every_msec", every_msec);
+	REQUIRE_PARAM_STRING("timer_id", timer_id);
+
+	subrequest = json_object_get(params, "request");
+	if (!subrequest)
+	{
+		rpc_error_fmt(client, request, JSON_RPC_ERROR_INVALID_PARAMS, "Missing parameter: '%s'", "request");
+		return;
+	}
+
+	if (every_msec < RPC_MINIMUM_TIMER_MSEC)
+	{
+		rpc_error_fmt(client, request, JSON_RPC_ERROR_INVALID_PARAMS,
+		              "Value for every_msec may not be less than %d",
+		              (int)RPC_MINIMUM_TIMER_MSEC);
+		return;
+	}
+
+	/* Do some validation on the name */
+	if (!parse_rpc_call(client, request, subrequest, &method, &handler))
+		return; /* Error already returned to caller */
+
+	/* We don't validate 'params' here, but do so at runtime */
+
+	/* Check if a timer with that same name already exists FOR THIS CLIENT */
+	if (find_rpc_timer(client, timer_id))
+	{
+		rpc_error_fmt(client, request, JSON_RPC_ERROR_ALREADY_EXISTS, "Timer already exists with timer_id '%s'", timer_id);
+		return;
+	}
+
+	timer = safe_alloc(sizeof(RPCTimer));
+	timer->every_msec = every_msec;
+	timer->client = client;
+	safe_strdup(timer->timer_id, timer_id);
+	json_incref(subrequest);
+	timer->request = subrequest;
+	AddListItem(timer, rpc_timer_list);
+	result = json_boolean(1);
+	rpc_response(client, request, result);
+	json_decref(result);
+}
+
+EVENT(rpc_do_timers)
+{
+	RPCTimer *e, *e_next;
+
+	for (e = rpc_timer_list; e; e = e_next)
+	{
+		e_next = e->next;
+		if (minimum_msec_since_last_run(&e->last_run, e->every_msec))
+		{
+			rpc_call_json(e->client, e->request);
+		}
+		// TODO: maybe do counts as well?
+	}
+}
+
+/** Client being freed? If RPC then cancel timers, if any */
+int rpc_handle_free_client(Client *client)
+{
+	if (IsRPC(client))
+		free_rpc_timers_for_user(client);
+	return 0;
+}
+
+RPC_CALL_FUNC(rpc_rpc_del_timer)
+{
+	const char *timer_id;
+	RPCTimer *r;
+	json_t *result;
+
+	REQUIRE_PARAM_STRING("timer_id", timer_id);
+
+	r = find_rpc_timer(client, timer_id);
+	if (!r)
+	{
+		rpc_error_fmt(client, request, JSON_RPC_ERROR_NOT_FOUND, "Timer not found with timer_id '%s'", timer_id);
+		return;
+	}
+	free_rpc_timer(r);
+
+	result = json_boolean(1);
+	rpc_response(client, request, result);
+	json_decref(result);
+}
