@@ -56,7 +56,6 @@ extern MODVAR int  max_connection_count;
 
 int stats_banversion(Client *, const char *);
 int stats_links(Client *, const char *);
-int stats_denylinkall(Client *, const char *);
 int stats_gline(Client *, const char *);
 int stats_except(Client *, const char *);
 int stats_allow(Client *, const char *);
@@ -67,7 +66,6 @@ int stats_bannick(Client *, const char *);
 int stats_traffic(Client *, const char *);
 int stats_uline(Client *, const char *);
 int stats_vhost(Client *, const char *);
-int stats_denylinkauto(Client *, const char *);
 int stats_kline(Client *, const char *);
 int stats_banrealname(Client *, const char *);
 int stats_sqline(Client *, const char *);
@@ -85,6 +83,8 @@ int stats_class(Client *, const char *);
 int stats_officialchannels(Client *, const char *);
 int stats_spamfilter(Client *, const char *);
 int stats_fdtable(Client *, const char *);
+int stats_linecache(Client *client, const char *para);
+int stats_maxperip(Client *, const char *);
 
 #define SERVER_AS_PARA 0x1
 #define FLAGS_AS_PARA 0x2
@@ -101,7 +101,6 @@ struct statstab {
 struct statstab StatsTable[] = {
 	{ 'B', "banversion",	stats_banversion,	0		},
 	{ 'C', "link", 		stats_links,		0 		},
-	{ 'D', "denylinkall",	stats_denylinkall,	0		},
 	{ 'G', "gline",		stats_gline,		FLAGS_AS_PARA	},
 	{ 'H', "link",	 	stats_links,		0 		},
 	{ 'I', "allow",		stats_allow,		0 		},
@@ -119,7 +118,6 @@ struct statstab StatsTable[] = {
 	{ 'X', "notlink",	stats_notlink,		0 		},
 	{ 'Y', "class",		stats_class,		0 		},
 	{ 'c', "link", 		stats_links,		0 		},
-	{ 'd', "denylinkauto",	stats_denylinkauto,	0 		},
 	{ 'e', "except",	stats_except,		0 		},
 	{ 'f', "spamfilter",	stats_spamfilter,	FLAGS_AS_PARA	},
 	{ 'g', "gline",		stats_gline,		FLAGS_AS_PARA	},
@@ -138,6 +136,8 @@ struct statstab StatsTable[] = {
 	{ 'v', "denyver",	stats_denyver,		0 		},
 	{ 'x', "notlink",	stats_notlink,		0 		},
 	{ 'y', "class",		stats_class,		0 		},
+	{ '8', "maxperip",	stats_maxperip,		0		},
+	{ '9', "linecache",	stats_linecache,	0		},
 	{ 0, 	NULL, 		NULL, 			0		}
 };
 
@@ -444,22 +444,6 @@ int stats_links(Client *client, const char *para)
 	return 0;
 }
 
-int stats_denylinkall(Client *client, const char *para)
-{
-	ConfigItem_deny_link *links;
-	ConfigItem_mask *m;
-
-	for (links = conf_deny_link; links; links = links->next)
-	{
-		if (links->flag.type == CRULE_ALL)
-		{
-			for (m = links->mask; m; m = m->next)
-				sendnumeric(client, RPL_STATSDLINE, 'D', m->mask, links->prettyrule);
-		}
-	}
-	return 0;
-}
-
 int stats_gline(Client *client, const char *para)
 {
 	int cnt = 0;
@@ -686,22 +670,6 @@ int stats_vhost(Client *client, const char *para)
 	return 0;
 }
 
-int stats_denylinkauto(Client *client, const char *para)
-{
-	ConfigItem_deny_link *links;
-	ConfigItem_mask *m;
-
-	for (links = conf_deny_link; links; links = links->next)
-	{
-		if (links->flag.type == CRULE_AUTO)
-		{
-			for (m = links->mask; m; m = m->next)
-				sendnumeric(client, RPL_STATSDLINE, 'd', m->mask, links->prettyrule);
-		}
-	}
-	return 0;
-}
-
 int stats_kline(Client *client, const char *para)
 {
 	int cnt = 0;
@@ -774,12 +742,14 @@ static void stats_set_anti_flood(Client *client, FloodSettings *f)
 
 	for (i=0; floodoption_names[i]; i++)
 	{
+		if (f->limit[i] == 0)
+			continue; /* unconfigured */
 		if (i == FLD_CONVERSATIONS)
 		{
 			sendtxtnumeric(client, "anti-flood::%s::%s: %d users, new user every %s",
 				f->name, floodoption_names[i],
 				(int)f->limit[i], pretty_time_val(f->period[i]));
-		}
+		} else
 		if (i == FLD_LAG_PENALTY)
 		{
 			sendtxtnumeric(client, "anti-flood::%s::lag-penalty: %d msec",
@@ -787,8 +757,7 @@ static void stats_set_anti_flood(Client *client, FloodSettings *f)
 			sendtxtnumeric(client, "anti-flood::%s::lag-penalty-bytes: %d",
 				f->name,
 				f->limit[i] == INT_MAX ? 0 : (int)f->limit[i]);
-		}
-		else
+		} else
 		{
 			sendtxtnumeric(client, "anti-flood::%s::%s: %d per %s",
 				f->name, floodoption_names[i],
@@ -831,7 +800,7 @@ int stats_set(Client *client, const char *para)
 	sendtxtnumeric(client, "kline-address: %s", KLINE_ADDRESS);
 	if (GLINE_ADDRESS)
 		sendtxtnumeric(client, "gline-address: %s", GLINE_ADDRESS);
-	sendtxtnumeric(client, "modes-on-connect: %s", get_usermode_string_raw(CONN_MODES));
+	//sendtxtnumeric(client, "modes-on-connect: %s", get_usermode_string_raw(CONN_MODES));
 	sendtxtnumeric(client, "modes-on-oper: %s", get_usermode_string_raw(OPER_MODES));
 	*modebuf = *parabuf = 0;
 	chmode_str(&iConf.modes_on_join, modebuf, parabuf, sizeof(modebuf), sizeof(parabuf));
@@ -845,8 +814,8 @@ int stats_set(Client *client, const char *para)
 		char *longflags = allow_user_stats_long_to_short();
 		sendtxtnumeric(client, "allow-user-stats: %s%s", ALLOW_USER_STATS, longflags ? longflags : "");
 	}
-	if (RESTRICT_USERMODES)
-		sendtxtnumeric(client, "restrict-usermodes: %s", RESTRICT_USERMODES);
+//	if (RESTRICT_USERMODES)
+//		sendtxtnumeric(client, "restrict-usermodes: %s", RESTRICT_USERMODES);
 	if (RESTRICT_CHANNELMODES)
 		sendtxtnumeric(client, "restrict-channelmodes: %s", RESTRICT_CHANNELMODES);
 	if (RESTRICT_EXTENDEDBANS)
@@ -886,12 +855,12 @@ int stats_set(Client *client, const char *para)
 	sendtxtnumeric(client, "options::mkpasswd-for-everyone: %d", MKPASSWD_FOR_EVERYONE);
 	sendtxtnumeric(client, "options::allow-insane-bans: %d", ALLOW_INSANE_BANS);
 	sendtxtnumeric(client, "options::allow-part-if-shunned: %d", ALLOW_PART_IF_SHUNNED);
-	sendtxtnumeric(client, "maxchannelsperuser: %i", MAXCHANNELSPERUSER);
+	//sendtxtnumeric(client, "maxchannelsperuser: %i", iConf.maxchannelsperuser);
 	sendtxtnumeric(client, "ping-warning: %i seconds", PINGWARNING);
-	sendtxtnumeric(client, "auto-join: %s", AUTO_JOIN_CHANS ? AUTO_JOIN_CHANS : "0");
+	//sendtxtnumeric(client, "auto-join: %s", AUTO_JOIN_CHANS ? AUTO_JOIN_CHANS : "0");
 	sendtxtnumeric(client, "oper-auto-join: %s", OPER_AUTO_JOIN_CHANS ? OPER_AUTO_JOIN_CHANS : "0");
-	sendtxtnumeric(client, "static-quit: %s", STATIC_QUIT ? STATIC_QUIT : "<none>");
-	sendtxtnumeric(client, "static-part: %s", STATIC_PART ? STATIC_PART : "<none>");
+	//sendtxtnumeric(client, "static-quit: %s", STATIC_QUIT ? STATIC_QUIT : "<none>");
+	//sendtxtnumeric(client, "static-part: %s", STATIC_PART ? STATIC_PART : "<none>");
 	sendtxtnumeric(client, "who-limit: %d", WHOLIMIT);
 	sendtxtnumeric(client, "silence-limit: %d", SILENCE_LIMIT);
 	sendtxtnumeric(client, "ban-version-tkl-time: %s", pretty_time_val(BAN_VERSION_TKL_TIME));
@@ -1091,3 +1060,66 @@ int stats_linkinfoint(Client *client, const char *para, int all)
 #endif
 	return 0;
 }
+
+int stats_linecache(Client *client, const char *para)
+{
+	ClientCapability *e;
+
+	if (!ValidatePermissionsForPath("server:info:stats",client,NULL,NULL,NULL))
+	{
+		sendnumeric(client, ERR_NOPRIVILEGES);
+		return 0;
+	}
+
+	sendtxtnumeric(client, "Line cache: caps that have an effect message tags:");
+	for (e = clicaps; e; e = e->next)
+		if (e->cap & clicaps_affecting_mtag)
+			sendtxtnumeric(client, "CAP %s", e->name);
+
+	//sendtxtnumeric(client, " ");
+	//sendtxtnumeric(client, "Line statistics:");	
+
+	return 0;
+}
+
+int stats_maxperip(Client *client, const char *para)
+{
+	int i;
+	IpUsersBucket *e;
+	char ipbuf[256];
+	const char *ip;
+
+	if (!ValidatePermissionsForPath("server:info:stats",client,NULL,NULL,NULL))
+	{
+		sendnumeric(client, ERR_NOPRIVILEGES);
+		return 0;
+	}
+
+	sendtxtnumeric(client, "MaxPerIp IPv4 hash table:");
+	for (i=0; i < IPUSERS_HASH_TABLE_SIZE; i++)
+	{
+		for (e = IpUsersHash_ipv4[i]; e; e = e->next)
+		{
+			ip = inetntop(AF_INET, e->rawip, ipbuf, sizeof(ipbuf));
+			if (!ip)
+				ip = "<invalid>";
+			sendtxtnumeric(client, "IPv4 #%d %s: %d local / %d global",
+				       i, ip, e->local_clients, e->global_clients);
+		}
+	}
+
+	sendtxtnumeric(client, "MaxPerIp IPv6 hash table:");
+	for (i=0; i < IPUSERS_HASH_TABLE_SIZE; i++)
+	{
+		for (e = IpUsersHash_ipv6[i]; e; e = e->next)
+		{
+			ip = inetntop(AF_INET6, e->rawip, ipbuf, sizeof(ipbuf));
+			if (!ip)
+				ip = "<invalid>";
+			sendtxtnumeric(client, "IPv6 #%d %s: %d local / %d global",
+				       i, ip, e->local_clients, e->global_clients);
+		}
+	}
+
+	return 0;
+}
