@@ -28,21 +28,11 @@ ModuleHeader MOD_HEADER
 #endif
 
 #define WSU(client)	((WebSocketUser *)moddata_client(client, websocket_md).ptr)
+#define WEB(client)		((WebRequest *)moddata_client(client, webserver_md).ptr)
 
 #define WEBSOCKET_PORT(client)	((client->local && client->local->listener) ? client->local->listener->websocket_options : 0)
 #define WEBSOCKET_TYPE(client)	(WSU(client)->type)
 
-/* used to parse http Forwarded header (RFC 7239) */
-#define IPLEN 48
-#define FHEADER_NAMELEN	20
-
-struct HTTPForwardedHeader
-{
-	int secure;
-	char hostname[HOSTLEN+1];
-	char ip[IPLEN+1];
-};
-
 /* Forward declarations */
 int websocket_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs);
 int websocket_config_posttest(int *);
@@ -52,12 +42,12 @@ int websocket_handle_handshake(Client *client, const char *readbuf, int *length)
 int websocket_handshake_send_response(Client *client);
 int websocket_handle_body_websocket(Client *client, WebRequest *web, const char *readbuf2, int length2);
 int websocket_secure_connect(Client *client);
-struct HTTPForwardedHeader *websocket_parse_forwarded_header(char *input);
-int websocket_ip_compare(const char *ip1, const char *ip2);
 int websocket_handle_request(Client *client, WebRequest *web);
+int websocket_reconfigure_web_listener(ConfigItem_listen *listener);
 
 /* Global variables */
-ModDataInfo *websocket_md;
+ModDataInfo *websocket_md = NULL; /* (by us) */
+ModDataInfo *webserver_md = NULL; /* (external module, looked up) */
 static int ws_text_mode_available = 1;
 
 MOD_TEST()
@@ -83,6 +73,7 @@ MOD_INIT()
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGRUN_EX, 0, websocket_config_run_ex);
 	HookAdd(modinfo->handle, HOOKTYPE_PACKET, INT_MAX, websocket_packet_out);
 	HookAdd(modinfo->handle, HOOKTYPE_SECURE_CONNECT, 0, websocket_secure_connect);
+	HookAdd(modinfo->handle, HOOKTYPE_RECONFIGURE_WEB_LISTENER, 0, websocket_reconfigure_web_listener);
 
 	/* Call MOD_LOAD very late, since we manage sockets, but depend on websocket_common */
 	ModuleSetOptions(modinfo->handle, MOD_OPT_PRIORITY, WEBSOCKET_MODULE_PRIORITY_UNLOAD-1);
@@ -91,6 +82,8 @@ MOD_INIT()
 
 MOD_LOAD()
 {
+	webserver_md = findmoddata_byname("web", MODDATATYPE_CLIENT);
+
 	if (non_utf8_nick_chars_in_use || (iConf.allowed_channelchars == ALLOWED_CHANNELCHARS_ANY))
 		ws_text_mode_available = 0;
 	return MOD_SUCCESS;
@@ -147,18 +140,10 @@ int websocket_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 			}
 		} else if (!strcmp(cep->name, "forward"))
 		{
-			if (!cep->value)
-			{
-				config_error_empty(cep->file->filename, cep->line_number, "listen::options::websocket::forward", cep->name);
-				errors++;
-				continue;
-			}
-			if (!is_valid_ip(cep->value))
-			{
-				config_error("%s:%i: invalid IP address '%s' in listen::options::websocket::forward", cep->file->filename, cep->line_number, cep->value);
-				errors++;
-				continue;
-			}
+			config_error("%s:%i: this functionality has been moved to the proxy { } block. "
+			             "See https://www.unrealircd.org/docs/Proxy_block",
+			             cep->file->filename, cep->line_number);
+			errors++;
 		} else
 		{
 			config_error("%s:%i: unknown directive listen::options::websocket::%s",
@@ -217,14 +202,21 @@ int websocket_config_run_ex(ConfigFile *cf, ConfigEntry *ce, int type, void *ptr
 					warned_once_channel = 1;
 				}
 			}
-		} else if (!strcmp(cep->name, "forward"))
-		{
-			safe_strdup(l->websocket_forward, cep->value);
 		}
 	}
 	return 1;
 }
 
+int websocket_reconfigure_web_listener(ConfigItem_listen *listener)
+{
+	if (listener->websocket_options)
+	{
+		listener->webserver->handle_request = websocket_handle_request;
+		listener->webserver->handle_body = websocket_handle_body_websocket;
+	}
+	return 0;
+}
+
 int websocket_config_posttest(int *errs)
 {
 	int errors = 0;
@@ -257,7 +249,7 @@ int websocket_config_posttest(int *errs)
 /* Add LF (if needed) to a buffer. Max 4K. */
 void add_lf_if_needed(char **buf, int *len)
 {
-	static char newbuf[4096];
+	static char newbuf[MAXLINELENGTH];
 	char *b = *buf;
 	int l = *len;
 
@@ -318,141 +310,6 @@ int websocket_packet_out(Client *from, Client *to, Client *intended_to, char **m
 	return 0;
 }
 
-#define FHEADER_STATE_NAME	0
-#define FHEADER_STATE_VALUE	1
-#define FHEADER_STATE_VALUE_QUOTED	2
-
-#define FHEADER_ACTION_APPEND	0
-#define FHEADER_ACTION_IGNORE	1
-#define FHEADER_ACTION_PROCESS	2
-
-/** If a valid Forwarded: http header is received from a trusted source (proxy server), this function will
-  * extract remote IP address and secure (https) status from it. If more than one field with same name is received,
-  * we'll accept the last one. This should work correctly with chained proxies. */
-struct HTTPForwardedHeader *websocket_parse_forwarded_header(char *input)
-{
-	static struct HTTPForwardedHeader forwarded;
-	int i, length;
-	int state = FHEADER_STATE_NAME, action = FHEADER_ACTION_APPEND;
-	char name[FHEADER_NAMELEN+1];
-	char value[IPLEN+1];
-	int name_length = 0;
-	int value_length = 0;
-	char c;
-	
-	memset(&forwarded, 0, sizeof(struct HTTPForwardedHeader));
-	
-	length = strlen(input);
-	for (i = 0; i < length; i++)
-	{
-		c = input[i];
-		switch (c)
-		{
-			case '"':
-				switch (state)
-				{
-					case FHEADER_STATE_NAME:
-						action = FHEADER_ACTION_APPEND;
-						break;
-					case FHEADER_STATE_VALUE:
-						action = FHEADER_ACTION_IGNORE;
-						state = FHEADER_STATE_VALUE_QUOTED;
-						break;
-					case FHEADER_STATE_VALUE_QUOTED:
-						action = FHEADER_ACTION_IGNORE;
-						state = FHEADER_STATE_VALUE;
-						break;
-				}
-				break;
-			case ',': case ';': case ' ':
-				switch (state)
-				{
-					case FHEADER_STATE_NAME: /* name without value */
-						name_length = 0;
-						action = FHEADER_ACTION_IGNORE;
-						break;
-					case FHEADER_STATE_VALUE: /* end of value */
-						action = FHEADER_ACTION_PROCESS;
-						break;
-					case FHEADER_STATE_VALUE_QUOTED: /* quoted character, process as normal */
-						action = FHEADER_ACTION_APPEND;
-						break;
-				}
-				break;
-			case '=':
-				switch (state)
-				{
-					case FHEADER_STATE_NAME: /* end of name */
-						name[name_length] = '\0';
-						state = FHEADER_STATE_VALUE;
-						action = FHEADER_ACTION_IGNORE;
-						break;
-					case FHEADER_STATE_VALUE: case FHEADER_STATE_VALUE_QUOTED: /* none of the values is expected to contain = but proceed anyway */
-						action = FHEADER_ACTION_APPEND;
-						break;
-				}
-				break;
-			default:
-				action = FHEADER_ACTION_APPEND;
-				break;
-		}
-		switch (action)
-		{
-			case FHEADER_ACTION_APPEND:
-				if (state == FHEADER_STATE_NAME)
-				{
-					if (name_length < FHEADER_NAMELEN)
-					{
-						name[name_length++] = c;
-					} else
-					{
-						/* truncate */
-					}
-				} else
-				{
-					if (value_length < IPLEN)
-					{
-						value[value_length++] = c;
-					} else
-					{
-						/* truncate */
-					}
-				}
-				break;
-			case FHEADER_ACTION_IGNORE: default:
-				break;
-			case FHEADER_ACTION_PROCESS:
-				value[value_length] = '\0';
-				name[name_length] = '\0';
-				if (!strcasecmp(name, "for"))
-				{
-					strlcpy(forwarded.ip, value, IPLEN+1);
-				} else if (!strcasecmp(name, "proto"))
-				{
-					if (!strcasecmp(value, "https"))
-					{
-						forwarded.secure = 1;
-					} else if (!strcasecmp(value, "http"))
-					{
-						forwarded.secure = 0;
-					} else
-					{
-						/* ignore unknown value */
-					}
-				} else
-				{
-					/* ignore unknown field name */
-				}
-				value_length = 0;
-				name_length = 0;
-				state = FHEADER_STATE_NAME;
-				break;
-		}
-	}
-	
-	return &forwarded;
-}
-
 /** We got a HTTP(S) request and we need to check if we can upgrade the connection
  * to a websocket connection.
  */
@@ -545,61 +402,9 @@ int websocket_handle_request(Client *client, WebRequest *web)
 		}
 	}
 
-	/* Check forwarded header (by k4be) */
-	if (WSU(client)->forwarded)
-	{
-		struct HTTPForwardedHeader *forwarded;
-		char oldip[64];
-
-		/* check for source ip */
-		if (BadPtr(client->local->listener->websocket_forward) || !websocket_ip_compare(client->local->listener->websocket_forward, client->ip))
-		{
-			unreal_log(ULOG_WARNING, "websocket", "UNAUTHORIZED_FORWARDED_HEADER", client, "Received unauthorized Forwarded header from $ip", log_data_string("ip", client->ip));
-			webserver_send_response(client, 403, "Forwarded: no access");
-			return 0;
-		}
-		/* parse the header */
-		forwarded = websocket_parse_forwarded_header(WSU(client)->forwarded);
-		/* check header values */
-		if (!is_valid_ip(forwarded->ip))
-		{
-			unreal_log(ULOG_WARNING, "websocket", "INVALID_FORWARDED_IP", client, "Received invalid IP in Forwarded header from $ip", log_data_string("ip", client->ip));
-			webserver_send_response(client, 400, "Forwarded: invalid IP");
-			return 0;
-		}
-		/* store data */
-		WSU(client)->secure = forwarded->secure;
-		strlcpy(oldip, client->ip, sizeof(oldip));
-		safe_strdup(client->ip, forwarded->ip);
-		/* Update client->local->hostp */
-		strlcpy(client->local->sockhost, forwarded->ip, sizeof(client->local->sockhost)); /* in case dns lookup fails or is disabled */
-		/* (free old) */
-		if (client->local->hostp)
-		{
-			unreal_free_hostent(client->local->hostp);
-			client->local->hostp = NULL;
-		}
-		/* (create new) */
-		if (!DONT_RESOLVE)
-		{
-			/* taken from socket.c */
-			struct hostent *he;
-			unrealdns_delreq_bycptr(client); /* in case the proxy ip is still in progress of being looked up */
-			ClearDNSLookup(client);
-			he = unrealdns_doclient(client); /* call this once more */
-			if (!client->local->hostp)
-			{
-				if (he)
-					client->local->hostp = he;
-				else
-					SetDNSLookup(client);
-			} else
-			{
-				/* Race condition detected, DNS has been done, continue with auth */
-			}
-		}
-		RunHook(HOOKTYPE_IP_CHANGE, client, oldip);
-	}
+	/* If using a proxy, set the secure flag depending on what the proxy said */
+	if (WEB(client)->forwarded)
+		WSU(client)->secure = WEB(client)->forwarded->secure;
 
 	websocket_handshake_send_response(client);
 	return 1;
@@ -656,33 +461,3 @@ int websocket_handshake_send_response(Client *client)
 
 	return 0;
 }
-
-/** Compare IP addresses (for authorization checking) */
-int websocket_ip_compare(const char *ip1, const char *ip2)
-{
-	uint32_t ip4[2];
-	uint16_t ip6[16];
-	int i;
-	if (inet_pton(AF_INET, ip1, &ip4[0]) == 1) /* IPv4 */
-	{
-		if (inet_pton(AF_INET, ip2, &ip4[1]) == 1) /* both are valid, let's compare */
-		{
-			return ip4[0] == ip4[1];
-		}
-		return 0;
-	}
-	if (inet_pton(AF_INET6, ip1, &ip6[0]) == 1) /* IPv6 */
-	{
-		if (inet_pton(AF_INET6, ip2, &ip6[8]) == 1)
-		{
-			for (i = 0; i < 8; i++)
-			{
-				if (ip6[i] != ip6[i+8])
-					return 0;
-			}
-			return 1;
-		}
-	}
-	return 0; /* neither valid IPv4 nor IPv6 */
-}
-
