@@ -61,6 +61,20 @@ void unreal_add_masks(ConfigItem_mask **head, ConfigEntry *ce)
 	}
 }
 
+ConfigItem_mask *unreal_duplicate_masks(ConfigItem_mask *existing)
+{
+	ConfigItem_mask *ret = NULL;
+	ConfigItem_mask *n;
+
+	for (; existing; existing = existing->next)
+	{
+		n = safe_alloc(sizeof(ConfigItem_mask));
+		safe_strdup(n->mask, existing->mask);
+		AddListItem(n, ret);
+	}
+	return ret;
+}
+
 /** Check if a client matches any of the masks in the mask list.
  * The following rules apply:
  * - If you have only negating entries, like '!abc' and '!def', then
@@ -168,6 +182,10 @@ int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors)
 	{
 		CheckNullX(cep);
 	} else
+	if (!strcmp(cep->name, "websocket") || !strcmp(cep->name, "exclude-websocket"))
+	{
+		CheckNullX(cep);
+	} else
 	if (!strcmp(cep->name, "identified") || !strcmp(cep->name, "exclude-identified"))
 	{
 		CheckNullX(cep);
@@ -365,6 +383,8 @@ int conf_match_item(ConfigFile *conf, ConfigEntry *cep, SecurityGroup **block)
 
 	if (!strcmp(cep->name, "webirc"))
 		s->webirc = config_checkval(cep->value, CFG_YESNO);
+	if (!strcmp(cep->name, "websocket"))
+		s->websocket = config_checkval(cep->value, CFG_YESNO);
 	else if (!strcmp(cep->name, "identified"))
 		s->identified = config_checkval(cep->value, CFG_YESNO);
 	else if (!strcmp(cep->name, "tls"))
@@ -397,6 +417,8 @@ int conf_match_item(ConfigFile *conf, ConfigEntry *cep, SecurityGroup **block)
 	}
 	else if (!strcmp(cep->name, "exclude-webirc"))
 		s->exclude_webirc = config_checkval(cep->value, CFG_YESNO);
+	else if (!strcmp(cep->name, "exclude-websocket"))
+		s->exclude_websocket = config_checkval(cep->value, CFG_YESNO);
 	else if (!strcmp(cep->name, "exclude-identified"))
 		s->exclude_identified = config_checkval(cep->value, CFG_YESNO);
 	else if (!strcmp(cep->name, "exclude-tls"))
@@ -558,6 +580,7 @@ SecurityGroup *add_security_group(const char *name, int priority)
 	s = safe_alloc(sizeof(SecurityGroup));
 	strlcpy(s->name, name, sizeof(s->name));
 	s->priority = priority;
+	init_dynamic_set_block(&s->settings);
 	AddListItemPrio(s, securitygroups, priority);
 	return s;
 }
@@ -567,6 +590,8 @@ void free_security_group(SecurityGroup *s)
 {
 	if (s == NULL)
 		return;
+	// IMPORTANT: if you add anything here,
+	// then also update duplicate_security_group() !!!!
 	unreal_delete_masks(s->mask);
 	unreal_delete_masks(s->exclude_mask);
 	free_entire_name_list(s->security_group);
@@ -576,9 +601,31 @@ void free_security_group(SecurityGroup *s)
 	free_nvplist(s->extended);
 	free_nvplist(s->exclude_extended);
 	free_nvplist(s->printable_list);
+	free_dynamic_set_block(&s->settings);
 	safe_free(s);
 }
 
+/** Duplicate a SecurityGroup struct */
+SecurityGroup *duplicate_security_group(SecurityGroup *s)
+{
+	SecurityGroup *n;
+
+	if (s == NULL)
+		return NULL;
+
+	n = safe_alloc(sizeof(SecurityGroup));
+	n->mask = unreal_duplicate_masks(s->mask);
+	n->exclude_mask = unreal_duplicate_masks(s->exclude_mask);
+	n->security_group = duplicate_name_list(s->security_group);
+	n->exclude_security_group = duplicate_name_list(s->exclude_security_group);
+	n->ip = duplicate_name_list(s->exclude_ip);
+	n->extended = duplicate_nvplist(s->extended);
+	n->exclude_extended = duplicate_nvplist(s->exclude_extended);
+	n->printable_list = duplicate_nvplist(s->printable_list);
+	// not duplicated since it makes no sense: s->settings
+	return n;
+}
+
 /** Initialize the default security-group blocks */
 void set_security_group_defaults(void)
 {
@@ -596,6 +643,10 @@ void set_security_group_defaults(void)
 	s = add_security_group("webirc-users", 50);
 	s->webirc = 1;
 
+	/* Default group: websocket */
+	s = add_security_group("websocket-users", 51);
+	s->websocket = 1;
+
 	/* Default group: known-users */
 	s = add_security_group("known-users", 100);
 	s->identified = 1;
@@ -725,6 +776,8 @@ int user_allowed_by_security_group(Client *client, SecurityGroup *s)
 		goto user_not_allowed;
 	if (s->exclude_webirc && moddata_client_get(client, "webirc"))
 		goto user_not_allowed;
+	if (s->exclude_websocket && moddata_client_get(client, "websocket"))
+		goto user_not_allowed;
 	if ((s->exclude_reputation_score > 0) && (GetReputation(client) >= s->exclude_reputation_score))
 		goto user_not_allowed;
 	if ((s->exclude_reputation_score < 0) && (GetReputation(client) < 0 - s->exclude_reputation_score))
@@ -753,6 +806,8 @@ int user_allowed_by_security_group(Client *client, SecurityGroup *s)
 		goto user_allowed;
 	if (s->webirc && moddata_client_get(client, "webirc"))
 		goto user_allowed;
+	if (s->websocket && moddata_client_get(client, "websocket"))
+		goto user_allowed;
 	if ((s->reputation_score > 0) && (GetReputation(client) >= s->reputation_score))
 		goto user_allowed;
 	if ((s->reputation_score < 0) && (GetReputation(client) < 0 - s->reputation_score))
