@@ -737,6 +737,25 @@ int hide_idle_time(Client *client, Client *target)
 	}
 }
 
+/** Get creation time of a client.
+ * @param client	The client to check (user, server, anything)
+ * @returns the time when the client first connected to IRC, or 0 for unknown.
+ */
+time_t get_creationtime(Client *client)
+{
+	const char *str;
+
+	/* Shortcut for local clients */
+	if (client->local)
+		return client->local->creationtime;
+
+	/* Otherwise, hopefully available through this... */
+	str = moddata_client_get(client, "creationtime");
+	if (!BadPtr(str) && (*str != '0'))
+		return atoll(str);
+	return 0;
+}
+
 /** Get how long a client is connected to IRC.
  * @param client	The client to check
  * @returns how long the client is connected to IRC (number of seconds)
@@ -829,6 +848,9 @@ void flood_limit_exceeded_log(Client *client, const char *floodname)
 {
 	char buf[1024];
 
+	// NOTE: If you ever change this format, there are a few more
+	// direct unreal_log() calls with "FLOOD_BLOCKED" in the file
+	// src/modules/targetfloodprot.c, so update those as well.
 	unreal_log(ULOG_INFO, "flood", "FLOOD_BLOCKED", client,
 	           "Flood blocked ($flood_type) from $client.details [$client.ip]",
 	           log_data_string("flood_type", floodname));
@@ -918,6 +940,7 @@ MODVAR const char *floodoption_names[] = {
 	"max-concurrent-conversations",
 	"lag-penalty",
 	"vhost-flood",
+	"max-channels-per-user",
 	NULL
 };
 
@@ -983,3 +1006,26 @@ const char *get_operclass(Client *client)
 	 */
 	return moddata_client_get(client, "operclass");
 }
+
+/* Yeah we should really start storing IP in raw form again... */
+struct sockaddr *raw_client_ip(Client *client)
+{
+	struct sockaddr *client_addr;
+	static struct sockaddr_in addr4;
+	static struct sockaddr_in6 addr6;
+
+	memset(&addr4, 0, sizeof(addr4));
+	memset(&addr6, 0, sizeof(addr6));
+	if (inet_pton(AF_INET, GetIP(client), &addr4.sin_addr.s_addr) == 1)
+	{
+		client_addr = (struct sockaddr *)&addr4;
+		client_addr->sa_family = AF_INET;
+		return client_addr;
+	} else if (inet_pton(AF_INET6, GetIP(client), &addr6.sin6_addr.s6_addr) == 1)
+	{
+		client_addr = (struct sockaddr *)&addr6;
+		client_addr->sa_family = AF_INET6;
+		return client_addr;
+	}
+	return NULL;
+}
