@@ -23,62 +23,84 @@
 // Consider making add_history an efunc? Or via a hook?
 // Some users may not want to load cmd_whowas at all.
 
-/* internally defined function */
-static void add_whowas_to_clist(WhoWas **, WhoWas *);
-static void del_whowas_from_clist(WhoWas **, WhoWas *);
-static void add_whowas_to_list(WhoWas **, WhoWas *);
-static void del_whowas_from_list(WhoWas **, WhoWas *);
+void add_whowas_to_clist(WhoWas **, WhoWas *);
+void del_whowas_from_clist(WhoWas **, WhoWas *);
+void add_whowas_to_list(WhoWas **, WhoWas *);
+void del_whowas_from_list(WhoWas **, WhoWas *);
 
 WhoWas MODVAR WHOWAS[NICKNAMEHISTORYLENGTH];
 WhoWas MODVAR *WHOWASHASH[WHOWAS_HASH_TABLE_SIZE];
 
 MODVAR int whowas_next = 0;
 
-void add_history(Client *client, int online)
+void free_whowas_fields(WhoWas *e)
+{
+	safe_free(e->name);
+	safe_free(e->hostname);
+	safe_free(e->virthost);
+	safe_free(e->realname);
+	safe_free(e->username);
+	safe_free(e->account);
+	safe_free(e->ip);
+	e->servername = NULL;
+	e->event = 0;
+	e->logon = 0;
+	e->logoff = 0;
+	e->connected_since = 0;
+
+	/* Remove from lists and reset hashv */
+	if (e->online)
+		del_whowas_from_clist(&(e->online->user->whowas), e);
+	del_whowas_from_list(&WHOWASHASH[e->hashv], e);
+	e->hashv = -1;
+}
+
+void create_whowas_entry(Client *client, WhoWas *e, WhoWasEvent event)
+{
+	e->hashv = hash_whowas_name(client->name);
+	e->event = event;
+	e->connected_since = get_creationtime(client);
+	e->logon = client->lastnick;
+	e->logoff = TStime();
+	e->umodes = client->umodes;
+	safe_strdup(e->name, client->name);
+	safe_strdup(e->username, client->user->username);
+	safe_strdup(e->hostname, client->user->realhost);
+	safe_strdup(e->ip, client->ip);
+	if (client->user->virthost)
+		safe_strdup(e->virthost, client->user->virthost);
+	else
+		safe_strdup(e->virthost, "");
+	e->servername = client->user->server;
+	safe_strdup(e->realname, client->info);
+	if (strcmp(client->user->account, "0"))
+		safe_strdup(e->account, client->user->account);
+
+	/* Its not string copied, a pointer to the scache hash is copied
+	   -Dianora
+	 */
+	/*  strlcpy(e->servername, client->user->server,HOSTLEN); */
+	e->servername = client->user->server;
+}
+
+void add_history(Client *client, int online, WhoWasEvent event)
 {
 	WhoWas *new;
 
 	new = &WHOWAS[whowas_next];
 
 	if (new->hashv != -1)
-	{
-		safe_free(new->name);
-		safe_free(new->hostname);
-		safe_free(new->virthost);
-		safe_free(new->realname);
-		safe_free(new->username);
-		new->servername = NULL;
+		free_whowas_fields(new);
 
-		if (new->online)
-			del_whowas_from_clist(&(new->online->user->whowas), new);
-		del_whowas_from_list(&WHOWASHASH[new->hashv], new);
-	}
-	new->hashv = hash_whowas_name(client->name);
-	new->logoff = TStime();
-	new->umodes = client->umodes;
-	safe_strdup(new->name, client->name);
-	safe_strdup(new->username, client->user->username);
-	safe_strdup(new->hostname, client->user->realhost);
-	if (client->user->virthost)
-		safe_strdup(new->virthost, client->user->virthost);
-	else
-		safe_strdup(new->virthost, "");
-	new->servername = client->user->server;
-	safe_strdup(new->realname, client->info);
-
-	/* Its not string copied, a pointer to the scache hash is copied
-	   -Dianora
-	 */
-	/*  strlcpy(new->servername, client->user->server,HOSTLEN); */
-	new->servername = client->user->server;
+	create_whowas_entry(client, new, event);
 
 	if (online)
 	{
 		new->online = client;
 		add_whowas_to_clist(&(client->user->whowas), new);
-	}
-	else
+	} else {
 		new->online = NULL;
+	}
 	add_whowas_to_list(&WHOWASHASH[new->hashv], new);
 	whowas_next++;
 	if (whowas_next == NICKNAMEHISTORYLENGTH)
@@ -149,7 +171,7 @@ void initwhowas()
 		WHOWASHASH[i] = NULL;
 }
 
-static void add_whowas_to_clist(WhoWas ** bucket, WhoWas * whowas)
+void add_whowas_to_clist(WhoWas ** bucket, WhoWas * whowas)
 {
 	whowas->cprev = NULL;
 	if ((whowas->cnext = *bucket) != NULL)
@@ -157,7 +179,7 @@ static void add_whowas_to_clist(WhoWas ** bucket, WhoWas * whowas)
 	*bucket = whowas;
 }
 
-static void del_whowas_from_clist(WhoWas ** bucket, WhoWas * whowas)
+void del_whowas_from_clist(WhoWas ** bucket, WhoWas * whowas)
 {
 	if (whowas->cprev)
 		whowas->cprev->cnext = whowas->cnext;
@@ -167,7 +189,7 @@ static void del_whowas_from_clist(WhoWas ** bucket, WhoWas * whowas)
 		whowas->cnext->cprev = whowas->cprev;
 }
 
-static void add_whowas_to_list(WhoWas ** bucket, WhoWas * whowas)
+void add_whowas_to_list(WhoWas ** bucket, WhoWas * whowas)
 {
 	whowas->prev = NULL;
 	if ((whowas->next = *bucket) != NULL)
@@ -175,7 +197,7 @@ static void add_whowas_to_list(WhoWas ** bucket, WhoWas * whowas)
 	*bucket = whowas;
 }
 
-static void del_whowas_from_list(WhoWas ** bucket, WhoWas * whowas)
+void del_whowas_from_list(WhoWas ** bucket, WhoWas * whowas)
 {
 	if (whowas->prev)
 		whowas->prev->next = whowas->next;
